/*
 ## Cypress USB 3.0 Platform source file (cyfxuvcdscr.c)
 ## ===========================
 ##
 ##  Copyright Cypress Semiconductor Corporation, 2010-2018,
 ##  All Rights Reserved
 ##  UNPUBLISHED, LICENSED SOFTWARE.
 ##
 ##  CONFIDENTIAL AND PROPRIETARY INFORMATION
 ##  WHICH IS THE PROPERTY OF CYPRESS.
 ##
 ##  Use of this file is governed
 ##  by the license agreement included in the file
 ##
 ##     <install>/license/license.txt
 ##
 ##  where <install> is the Cypress software
 ##  installation root directory path.
 ##
 ## ===========================
*/

/* This file contains the USB enumeration descriptors for the UVC (in memory) application example.
 * The descriptor arrays must be 32 byte aligned and multiple of 32 bytes if the D-cache is
 * turned on. If the linker used is not capable of supporting the aligned feature for this,
 * either the descriptors must be placed in a different section and the section should be 
 * 32 byte aligned and 32 byte multiple; or dynamically allocated buffer allocated using
 * CyU3PDmaBufferAlloc must be used, and the descriptor must be loaded into it. The example
 * assumes that the aligned attribute for 32 bytes is supported by the linker. Do not add
 * any other variables to this file other than USB descriptors. This is not the only
 * pre-requisite to enabling the D-cache. Refer to the documentation for
 * CyU3PDeviceCacheControl for more information.
 */
#ifndef _CYFXYVCDESCR_H_
#define _CYFXYVCDESCR_H_


#include "cyfxuvcinmem.h"

/* Standard device descriptor for USB 3.0 */
const uint8_t CyFxUSB30DeviceDscr[] __attribute__ ((aligned (32))) =
{
    0x12,                           /* Descriptor size */
    CY_U3P_USB_DEVICE_DESCR,        /* Device descriptor type */
    0x10,0x03,                      /* USB 3.1 */
    0xEF,                           /* Device class */
    0x02,                           /* Device Sub-class */
    0x01,                           /* Device protocol */
    0x09,                           /* Maxpacket size for EP0 : 2^9 */
    0xB4,0x04,                      /* Vendor ID */
    0x22,0x47,                      /* Product ID */
    0x00,0x00,                      /* Device release number */
    0x01,                           /* Manufacture string index */
    0x02,                           /* Product string index */
    0x00,                           /* Serial number string index */
    0x01                            /* Number of configurations */
};

/* Standard device descriptor */
const uint8_t CyFxUSB20DeviceDscr[] __attribute__ ((aligned (32))) =
{
    0x12,                           /* Descriptor size */
    CY_U3P_USB_DEVICE_DESCR,        /* Device descriptor type */
    0x10,0x02,                      /* USB 2.10 */
    0xEF,                           /* Device class */
    0x02,                           /* Device sub-class */
    0x01,                           /* Device protocol */
    0x40,                           /* Maxpacket size for EP0 : 64 bytes */
    0xB4,0x04,                      /* Vendor ID */
    0x22,0x47,                      /* Product ID */
    0x00,0x00,                      /* Device release number */
    0x01,                           /* Manufacture string index */
    0x02,                           /* Product string index */
    0x00,                           /* Serial number string index */
    0x01                            /* Number of configurations */
};

/* Binary device object store descriptor */
const uint8_t CyFxUSBBOSDscr[] __attribute__ ((aligned (32))) =
{
    0x05,                           /* Descriptor size */
    CY_U3P_BOS_DESCR,               /* Device descriptor type */
    0x16,0x00,                      /* Length of this descriptor and all sub descriptors */
    0x02,                           /* Number of device capability descriptors */

    /* USB 2.0 Extension */
    0x07,                           /* Descriptor size */
    CY_U3P_DEVICE_CAPB_DESCR,       /* Device capability type descriptor */
    CY_U3P_USB2_EXTN_CAPB_TYPE,     /* USB 2.0 extension capability type */
    0x1E,0x64,0x00,0x00,            /* Supported device level features: LPM support, BESL supported,
                                       Baseline BESL=400 us, Deep BESL=1000 us. */

    /* SuperSpeed Device Capability */
    0x0A,                           /* Descriptor size */
    CY_U3P_DEVICE_CAPB_DESCR,       /* Device capability type descriptor */
    CY_U3P_SS_USB_CAPB_TYPE,        /* SuperSpeed device capability type */
    0x00,                           /* Supported device level features  */
    0x0E,0x00,                      /* Speeds supported by the device : SS, HS and FS */
    0x03,                           /* Functionality support */
    0x00,                           /* U1 device exit latency */
    0x00,0x00                       /* U2 device exit latency */
};

/* Standard device qualifier descriptor */
const uint8_t CyFxUSBDeviceQualDscr[] __attribute__ ((aligned (32))) =
{
    0x0A,                           /* descriptor size */
    CY_U3P_USB_DEVQUAL_DESCR,       /* Device qualifier descriptor type */
    0x00,0x02,                      /* USB 2.0 */
    0xEF,                           /* Device class */
    0x02,                           /* Device sub-class */
    0x01,                           /* Device protocol */
    0x40,                           /* Maxpacket size for EP0 : 64 bytes */
    0x01,                           /* Number of configurations */
    0x00                            /* Reserved */
};

//#define ORIG

/* Standard super speed configuration descriptor */
const uint8_t CyFxUSBSSConfigDscr[] __attribute__ ((aligned (32))) =
{
    /* Configuration descriptor */
    0x09,                           /* Descriptor size */
    CY_U3P_USB_CONFIG_DESCR,        /* Configuration descriptor type */
    0x3f,0x01,                      /* Length of this descriptor and all sub descriptors */
    0x05,                           /* Number of interfaces */
    0x01,                           /* Configuration number */
    0x00,                           /* COnfiguration string index */
    0x80,                           /* Config characteristics - Bus powered */
    0x32,                           /* Max power consumption of device (in 8mA unit) : 400mA */

    /* Interface association descriptor */
    0x08,                           /* Descriptor size */
    CY_FX_INTF_ASSN_DSCR_TYPE,      /* Interface association descr type */
    0x00,                           /* Interface number of first VideoControl i/f */
    0x02,                           /* Number of video i/f */
    0x0E,                           /* CC_VIDEO : Video i/f class code */
    0x03,                           /* SC_VIDEO_INTERFACE_COLLECTION : Subclass code */
    0x00,                           /* Protocol : Not used */
    0x00,                           /* String desc index for interface */

    /* Standard video control interface descriptor */
    0x09,                           /* Descriptor size */
    CY_U3P_USB_INTRFC_DESCR,        /* Interface descriptor type */
    0x00,                           /* Interface number */
    0x00,                           /* Alternate setting number */
    0x01,                           /* Number of end points */
    0x0E,                           /* CC_VIDEO : Interface class */
    0x01,                           /* CC_VIDEOCONTROL : Interface sub class */
    0x00,                           /* Interface protocol code */
    0x00,                           /* Interface descriptor string index */

    /* Class specific VC interface header descriptor */
    0x0D,                           /* Descriptor size */
    0x24,                           /* Class specific i/f header descriptor type */
    0x01,                           /* Descriptor sub type : VC_HEADER */
    0x10,0x01,                      /* Revision of class spec : 1.1 */
    0x51,0x00,                      /* Total size of class specific descriptors (till output terminal) */
    0x00,0x6C,0xDC,0x02,            /* Clock frequency : 48MHz */
    0x01,                           /* Number of streaming interfaces */
    0x01,                           /* Video streaming i/f 1 belongs to VC i/f */

    /* Input (camera) terminal descriptor */
    0x12,                           /* Descriptor size */
    0x24,                           /* Class specific interface desc type */
    0x02,                           /* Input Terminal Descriptor type */
    0x01,                           /* ID of this terminal */
    0x01,0x02,                      /* Camera terminal type */
    0x00,                           /* No association terminal */
    0x00,                           /* String desc index : Not used */
    0x00,0x00,                      /* No optical zoom supported */
    0x00,0x00,                      /* No optical zoom supported */
    0x00,0x00,                      /* No optical zoom supported */
    0x03,                           /* Size of controls field for this terminal : 3 bytes */
    0x00,0x00,0x00,                 /* No controls supported */

    /* Processing unit descriptor */
    0x0D,                           /* Descriptor size */
    0x24,                           /* Class specific interface desc type */
    0x05,                           /* Processing unit descriptor type */
    0x02,                           /* ID of this terminal */
    0x01,                           /* Source ID : 1 : connected to input terminal */
    0x00,0x40,                      /* Digital multiplier */
    0x03,                           /* Size of controls field for this terminal : 3 bytes */
    0x00,0x00,0x00,                 /* No controls supported */
    0x00,                           /* String desc index : Not used */
    0x00,                           /* No analog mode support. */

    /* Extension unit descriptor */
    0x1C,                           /* Descriptor size */
    0x24,                           /* Class specific interface desc type */
    0x06,                           /* Extension unit descriptor type */
    0x03,                           /* ID of this terminal */
    0xFF,0xFF,0xFF,0xFF,            /* 16 byte GUID */
    0xFF,0xFF,0xFF,0xFF,
    0xFF,0xFF,0xFF,0xFF,
    0xFF,0xFF,0xFF,0xFF,
    0x00,                           /* Number of controls in this terminal */
    0x01,                           /* Number of input pins in this terminal */
    0x02,                           /* Source ID : 2 : connected to proc unit */
    0x03,                           /* Size of controls field for this terminal : 3 bytes */
    0x00,0x00,0x00,                 /* No controls supported */
    0x00,                           /* String desc index : Not used */

    /* Output terminal descriptor */
    0x09,                           /* Descriptor size */
    0x24,                           /* Class specific interface desc type */
    0x03,                           /* Output terminal descriptor type */
    0x04,                           /* ID of this terminal */
    0x01,0x01,                      /* USB streaming terminal type */
    0x00,                           /* No association terminal */
    0x03,                           /* Source ID : 3 : Connected to extn Unit */
    0x00,                           /* String desc index : Not used */

    /* Video control status interrupt endpoint descriptor */
    0x07,                           /* Descriptor size */
    CY_U3P_USB_ENDPNT_DESCR,        /* Endpoint descriptor type */
    CY_FX_EP_CONTROL_STATUS,        /* Endpoint address and description */
    CY_U3P_USB_EP_INTR,             /* Interrupt end point type */
    0x40,0x00,                      /* Max packet size = 64 bytes */
    0x01,                           /* Servicing interval */

    /* Super speed endpoint companion descriptor */
    0x06,                           /* Descriptor size */
    CY_U3P_SS_EP_COMPN_DESCR,       /* SS endpoint companion descriptor type */
    0x00,                           /* Max no. of packets in a Burst : 1 */
    0x00,                           /* Mult.: Max number of packets : 1 */
    0x40, 0x00,                     /* Bytes per interval : 64 */

    /* Class specific interrupt endpoint descriptor */
    0x05,                           /* Descriptor size */
    0x25,                           /* Class specific endpoint descriptor type */
    CY_U3P_USB_EP_INTR,             /* End point sub type */
    0x40,0x00,                      /* Max packet size = 64 */

    /* Standard video streaming interface descriptor (Alternate setting 0) */
    0x09,                           /* Descriptor size */
    CY_U3P_USB_INTRFC_DESCR,        /* Interface descriptor type */
    0x01,                           /* Interface number */
    0x00,                           /* Alternate setting number */
    0x01,                           /* Number of end points : One bulk */
    0x0E,                           /* Interface class : CC_VIDEO */
    0x02,                           /* Interface sub class : CC_VIDEOSTREAMING */
    0x00,                           /* Interface protocol code : Undefined */
    0x00,                           /* Interface descriptor string index */
    /* Class-specific Video Streaming Input Header Descriptor */
    0x0E,                           /* Descriptor size */
    0x24,                           /* Class-specific VS I/f Type */
    0x01,                           /* Descriptotor Subtype : Input Header */
    0x01,                           /* 1 format desciptor follows */
    0x47,0x00,                      /* Total size of Class specific VS descr */
    CY_FX_EP_BULK_VIDEO,            /* EP address for BULK video data */
    0x00,                           /* No dynamic format change supported */
    0x04,                           /* Output terminal ID : 4 */
    0x00,                           /* Still image capture method 1 supported */
    0x00,                           /* Hardware trigger NOT supported */
    0x00,                           /* Hardware to initiate still image capture NOT supported */
    0x01,                           /* Size of controls field : 1 byte */
    0x00,                           /* D2 : Compression quality supported 14*/

    /* Class specific Uncompressed VS format descriptor */
    0x1B,                           /* Descriptor size */
    0x24,                           /* Class-specific VS I/f Type */
    0x04,                           /* Subtype : uncompressed format I/F */
    0x01,                           /* Format desciptor index */
    0x01,                           /* Number of frame descriptor followed */
    0x59,0x55,0x59,0x32,            /* GUID used to identify streaming-encoding format: YUY2  */
    0x00,0x00,0x10,0x00,
    0x80,0x00,0x00,0xAA,
    0x00,0x38,0x9B,0x71,
    IMAGE_BPP,                           /* Number of bits per pixel */
    0x01,                           /* Optimum Frame Index for this stream: 1 */
    0x10,                           /* X dimension of the picture aspect ratio; Non-interlaced */
    0x09,                           /* Y dimension of the pictuer aspect ratio: Non-interlaced */
    0x00,                           /* Interlace Flags: Progressive scanning, no interlace */
    0x00,                           /* duplication of the video stream restriction: 0 - no restriction 27 */

    /* Class specific Uncompressed VS frame descriptor */
    0x1E,                           /* Descriptor size */
    0x24,                           /* Descriptor type*/
    0x05,                           /* Subtype: uncompressed frame I/F */
    0x01,                           /* Frame Descriptor Index */
    0x01,                           /* Still image capture method 1 supported */
    IMAGE_WIDTH_800,                    /* Width in pixel */
    IMAGE_HEIGHT_482,                   /* Height in pixel */
    0x00,0x50,0x97,0x31,            /* Min bit rate bits/s. */
    0x00,0x50,0x97,0x31,            /* Max bit rate bits/s. */
    0x00,0xA4,0x1F,0x00,            /* Maximum video or still frame size in bytes(Deprecated)*/
    FPS_120,         /* 30fps */
    0x01,
    FPS_120,			//30

    /* Endpoint Descriptor for BULK Streaming Video Data */
    0x07,                           /* Descriptor size */
    CY_U3P_USB_ENDPNT_DESCR,        /* Endpoint Descriptor Type */
    CY_FX_EP_BULK_VIDEO,            /* Endpoint address and description */
    CY_U3P_USB_EP_BULK,             /* BULK End point */
    0x00, /* EP MaxPcktSize: 1024B */
    0x04, /* EP MaxPcktSize: 1024B */
    0x00,                           /* Servicing interval for data transfers */
    
    /* Super Speed Endpoint Companion Descriptor */
    0x06,                           /* Descriptor size */
    CY_U3P_SS_EP_COMPN_DESCR,       /* SS Endpoint Companion Descriptor Type */
    0x0F,                           /* Max number of packets per burst: 16 */
    0x00,                           /* Attribute: Streams not defined */
    0x00,                           /* No meaning for bulk */
    0x00,


    /* Interface association descriptor */
	0x08,                           /* Descriptor size */
	CY_FX_INTF_ASSN_DSCR_TYPE,      /* Interface association descr type */
	0x02,                           /* First interface number */
	0x02,                           /* i/f count */
	0x02,                           /* CC - CDC i/f class code */
	0x02,                           /* SC - Subclass code */
	0x01,                           /* Protocol : Not used */
	0x00,                           /* String desc index for interface */

    /* Communication Interface descriptor */
	0x09,                           /* Descriptor size */
	CY_U3P_USB_INTRFC_DESCR,        /* Interface Descriptor type */
	0x02,                           /* Interface number */
	0x00,                           /* Alternate setting number */
	0x01,                           /* Number of endpoints */
	0x02,                           /* Interface class : Communication interface */
	0x02,                           /* Interface sub class */
	0x01,                           /* Interface protocol code */
	0x00,                           /* Interface descriptor string index */

	/* CDC Class-specific Descriptors */
	/* Header functional Descriptor */
	0x05,                           /* Descriptors length(5) */
	0x24,                           /* Descriptor type : CS_Interface */
	0x00,                           /* DescriptorSubType : Header Functional Descriptor */
	0x10,0x01,                      /* bcdCDC : CDC Release Number */

	/* Abstract Control Management Functional Descriptor */
	0x04,                           /* Descriptors Length (4) */
	0x24,                           /* bDescriptorType: CS_INTERFACE */
	0x02,                           /* bDescriptorSubType: Abstract Control Model Functional Descriptor */
	0x02,                           /* bmCapabilities: Supports the request combination of Set_Line_Coding,
									   Set_Control_Line_State, Get_Line_Coding and the notification Serial_State */

	/* Union Functional Descriptor */
	0x05,                           /* Descriptors Length (5) */
	0x24,                           /* bDescriptorType: CS_INTERFACE */
	0x06,                           /* bDescriptorSubType: Union Functional Descriptor */
	0x00,                           /* bMasterInterface */
	0x01,                           /* bSlaveInterface */

	/* Call Management Functional Descriptor */
	0x05,                           /*  Descriptors Length (5) */
	0x24,                           /*  bDescriptorType: CS_INTERFACE */
	0x01,                           /*  bDescriptorSubType: Call Management Functional Descriptor */
	0x00,                           /*  bmCapabilities: Device sends/receives call management information
										only over the Communication Class Interface. */
	0x01,                           /*  Interface Number of Data Class interface */

	/* Endpoint Descriptor(Interrupt) */
	0x07,                           /* Descriptor size */
	CY_U3P_USB_ENDPNT_DESCR,        /* Endpoint descriptor type */
	CY_FX_EP_INTERRUPT,             /* Endpoint address and description */
	CY_U3P_USB_EP_INTR,             /* Interrupt endpoint type */
	0x40,0x00,                      /* Max packet size = 1024 bytes */
	0x01,                           /* Servicing interval for data transfers */

	/* Super speed endpoint companion descriptor for interrupt endpoint */
	0x06,                           /* Descriptor size */
	CY_U3P_SS_EP_COMPN_DESCR,       /* SS endpoint companion descriptor type */
	0x00,                           /* Max no. of packets in a Burst : 1 */
	0x00,                           /* Mult.: Max number of packets : 1 */
	0x40,0x00,                      /* Bytes per interval : 1024 */

	/* Data Interface Descriptor */
	0x09,                           /* Descriptor size */
	CY_U3P_USB_INTRFC_DESCR,        /* Interface Descriptor type */
	0x03,                           /* Interface number */
	0x00,                           /* Alternate setting number */
	0x02,                           /* Number of endpoints */
	0x0A,                           /* Interface class: Data interface */
	0x00,                           /* Interface sub class */
	0x00,                           /* Interface protocol code */
	0x00,                           /* Interface descriptor string index */

	/* Endpoint Descriptor(BULK-PRODUCER) */
	0x07,                           /* Descriptor size */
	CY_U3P_USB_ENDPNT_DESCR,        /* Endpoint descriptor type */
	CY_FX_EP_PRODUCER,              /* Endpoint address and description */
	CY_U3P_USB_EP_BULK,             /* BULK endpoint type */
	0x00,0x04,                      /* Max packet size = 1024 bytes */
	0x00,                           /* Servicing interval for data transfers */

	/* Super speed endpoint companion descriptor for producer ep */
	0x06,                           /* Descriptor size */
	CY_U3P_SS_EP_COMPN_DESCR,       /* SS endpoint companion descriptor type */
	0x00,                           /* Max no. of packets in a burst : 1 */
	0x00,                           /* Mult.: Max number of packets : 1 */
	0x00,0x00,                      /* Bytes per interval : 1024 */

	/* Endpoint Descriptor(BULK- CONSUMER) */
	0x07,                           /* Descriptor size */
	CY_U3P_USB_ENDPNT_DESCR,        /* Endpoint descriptor type */
	CY_FX_EP_CONSUMER,              /* Endpoint address and description */
	CY_U3P_USB_EP_BULK,             /* BULK endpoint type */
	0x00,0x04,                      /* Max packet size = 1024 bytes */
	0x00,                           /* Servicing interval for data transfers */

	/* Super speed endpoint companion descriptor for consumer ep */
	0x06,                           /* Descriptor size */
	CY_U3P_SS_EP_COMPN_DESCR,       /* SS endpoint companion descriptor type */
	0x00,                           /* Max no. of packets in a burst : 1 */
	0x00,                           /* Mult.: Max number of packets : 1 */
	0x00,0x00,                       /* Bytes per interval : 1024 */

	/* Interface association descriptor */
	0x08,                           /* Descriptor size */
	CY_FX_INTF_ASSN_DSCR_TYPE,      /* Interface association descr type */
	0x04,                           /* First interface number */
	0x01,                           /* Number of interfaces */
	0xFF,                           /* Class code */
	0x00,                           /* Subclass code */
	0x00,                           /* Protocol : Not used */
	0x01,                           /* String desc index for interface */

	/* Interface descriptor */
	0x09,                           /* Descriptor size */
	CY_U3P_USB_INTRFC_DESCR,        /* Interface Descriptor type */
	0x04,                           /* Interface number */
	0x00,                           /* Alternate setting number */
	0x00,                           /* Number of end points */
	0xFF,                           /* Interface class */
	0x00,                           /* Interface sub class */
	0x00,                           /* Interface protocol code */
	0x01,                           /* Interface descriptor string index */

};

/* Standard High Speed Configuration Descriptor */
uint8_t CyFxUSBHSConfigDscr[] __attribute__ ((aligned (32))) =
{
      /* Configuration descriptor */
    0x09,                           /* Descriptor size */
    CY_U3P_USB_CONFIG_DESCR,        /* Configuration descriptor type */
    0x5c,0x00,                      /* Length of this descriptor and all sub descriptors */
    0x03,                           /* Number of interfaces */
    0x01,                           /* Configuration number */
    0x00,                           /* COnfiguration string index */
    0x80,                           /* Config characteristics - bus powered */
    0xC8,                           /* Max power consumption of device (in 2mA unit) : 400mA */

    /* Interface association descriptor */
    	0x08,                           /* Descriptor size */
    	CY_FX_INTF_ASSN_DSCR_TYPE,      /* Interface association descr type */
    	0x00,                           /* First interface number */
    	0x02,                           /* i/f count */
    	0x02,                           /* CC - CDC i/f class code */
    	0x02,                           /* SC - Subclass code */
    	0x01,                           /* Protocol : Not used */
    	0x00,                           /* String desc index for interface */

        /* Communication Interface descriptor */
        0x09,                           /* Descriptor size */
        CY_U3P_USB_INTRFC_DESCR,        /* Interface Descriptor type */
        0x00,                           /* Interface number */
        0x00,                           /* Alternate setting number */
        0x01,                           /* Number of endpoints */
        0x02,                           /* Interface class : Communication Interface */
        0x02,                           /* Interface sub class */
        0x01,                           /* Interface protocol code */
        0x00,                           /* Interface descriptor string index */

        /* CDC Class-specific Descriptors */
        /* Header functional Descriptor */
        0x05,                           /* Descriptors length(5) */
        0x24,                           /* Descriptor type : CS_Interface */
        0x00,                           /* DescriptorSubType : Header Functional Descriptor */
        0x10,0x01,                      /* bcdCDC : CDC Release Number */

        /* Abstract Control Management Functional Descriptor */
        0x04,                           /* Descriptors Length (4) */
        0x24,                           /* bDescriptorType: CS_INTERFACE */
        0x02,                           /* bDescriptorSubType: Abstract Control Model Functional Descriptor */
        0x02,                           /* bmCapabilities: Supports the request combination of Set_Line_Coding,
                                           Set_Control_Line_State, Get_Line_Coding and the notification Serial_State */

        /* Union Functional Descriptor */
        0x05,                           /* Descriptors Length (5) */
        0x24,                           /* bDescriptorType: CS_INTERFACE */
        0x06,                           /* bDescriptorSubType: Union Functional Descriptor */
        0x00,                           /* bMasterInterface */
        0x01,                           /* bSlaveInterface */

        /* Call Management Functional Descriptor */
        0x05,                           /*  Descriptors Length (5) */
        0x24,                           /*  bDescriptorType: CS_INTERFACE */
        0x01,                           /*  bDescriptorSubType: Call Management Functional Descriptor */
        0x00,                           /*  bmCapabilities: Device sends/receives call management information
                                            only over the Communication Class Interface. */
        0x01,                           /*  Interface Number of Data Class interface */

        /* Endpoint Descriptor(Interrupt) */
        0x07,                           /* Descriptor size */
        CY_U3P_USB_ENDPNT_DESCR,        /* Endpoint descriptor type */
        0x81,                           /* Endpoint address and description */
        CY_U3P_USB_EP_INTR,             /* Interrupt endpoint type */
        0x40,0x00,                      /* Max packet size = 64 bytes */
        0x02,                           /* Servicing interval for data transfers */

        /* Data Interface Descriptor */
        0x09,                           /* Descriptor size */
        CY_U3P_USB_INTRFC_DESCR,        /* Interface Descriptor type */
        0x01,                           /* Interface number */
        0x00,                           /* Alternate setting number */
        0x02,                           /* Number of endpoints */
        0x0A,                           /* Interface class: Data interface */
        0x00,                           /* Interface sub class */
        0x00,                           /* Interface protocol code */
        0x00,                           /* Interface descriptor string index */

        /* Endpoint Descriptor(BULK-PRODUCER) */
        0x07,                           /* Descriptor size */
        CY_U3P_USB_ENDPNT_DESCR,        /* Endpoint descriptor type */
        CY_FX_EP_PRODUCER,                           /* Endpoint address and description */
        CY_U3P_USB_EP_BULK,             /* BULK endpoint type */
        0x00,0x02,                      /* Max packet size = 512 bytes */
        0x00,                           /* Servicing interval for data transfers */

        /* Endpoint Descriptor(BULK- CONSUMER) */
        0x07,                           /* Descriptor size */
        CY_U3P_USB_ENDPNT_DESCR,        /* Endpoint descriptor type */
        CY_FX_EP_CONSUMER,                           /* Endpoint address and description */
        CY_U3P_USB_EP_BULK,             /* Bulk endpoint type */
        0x00,0x02,                      /* Max packet size = 512 bytes */
        0x00,                           /* Servicing interval for data transfers */

	/* Interface association descriptor */
	0x08,                           /* Descriptor size */
	CY_FX_INTF_ASSN_DSCR_TYPE,      /* Interface association descr type */
	0x04,                           /* First interface number */
	0x01,                           /* Number of interfaces */
	0xFF,                           /* Class code */
	0x00,                           /* Subclass code */
	0x00,                           /* Protocol : Not used */
	0x01,                           /* String desc index for interface */

	/* Interface descriptor */
	0x09,                           /* Descriptor size */
	CY_U3P_USB_INTRFC_DESCR,        /* Interface Descriptor type */
	0x04,                           /* Interface number */
	0x00,                           /* Alternate setting number */
	0x00,                           /* Number of end points */
	0xFF,                           /* Interface class */
	0x00,                           /* Interface sub class */
	0x00,                           /* Interface protocol code */
	0x01                           /* Interface descriptor string index */

};

/* Standard full speed configuration descriptor : full speed is not supported. */
const uint8_t CyFxUSBFSConfigDscr[] __attribute__ ((aligned (32))) =
{
    /* Configuration descriptor */
    0x09,                           /* Descriptor size */
    CY_U3P_USB_CONFIG_DESCR,        /* Configuration descriptor type */
    0x09,0x00,                      /* Length of this descriptor and all sub descriptors */
    0x00,                           /* Number of interfaces */
    0x01,                           /* Configuration number */
    0x00,                           /* COnfiguration string index */
    0x80,                           /* Config characteristics - bus powered */
    0x32                            /* Max power consumption of device (in 2mA unit) : 100mA */
};

/* Standard language ID string descriptor */
const uint8_t CyFxUSBStringLangIDDscr[] __attribute__ ((aligned (32))) =
{
    0x04,                           /* Descriptor size */
    CY_U3P_USB_STRING_DESCR,        /* Device descriptor type */
    0x09,0x04                       /* Language ID supported */
};

/* Standard manufacturer string descriptor */
const uint8_t CyFxUSBManufactureDscr[] __attribute__ ((aligned (32))) =
{
    0x22,                           /* Descriptor size */
    CY_U3P_USB_STRING_DESCR,        /* Device descriptor type */
    'B',0x00,
    'r',0x00,
    'i',0x00,
    'g',0x00,
    'h',0x00,
    't',0x00,
    'W',0x00,
    'a',0x00,
    'y',0x00,
    ' ',0x00,
    'V',0x00,
    'i',0x00,
    's',0x00,
    'i',0x00,
    'o',0x00,
    'n',0x00,
};

/* Standard product string descriptor */
const uint8_t CyFxUSBProductDscr[] __attribute__ ((aligned (32))) =
{
    0x16,                           /* Descriptor Size */
    CY_U3P_USB_STRING_DESCR,        /* Device descriptor type */
    'B',0x00,
    'W',0x00,
    'V',0x00,
    ' ',0x00,
    'C',0x00,
    'a',0x00,
    'm',0x00,
    'e',0x00,
    'r',0x00,
    'a',0x00
};

const uint8_t CyFxUSBProductDscrProg[] __attribute__ ((aligned (32))) =
{
    0x18,                           /* Descriptor Size */
    CY_U3P_USB_STRING_DESCR,        /* Device descriptor type */
    'B',0x00,
    'W',0x00,
    'V',0x00,
    ' ',0x00,
    'P',0x00,
    'r',0x00,
    'o',0x00,
    'g',0x00,
    'r',0x00,
    'a',0x00,
    'm',0x00
};


/* Place this buffer as the last buffer so that no other variable / code shares
 * the same cache line. Do not add any other variables / arrays in this file.
 * This will lead to variables sharing the same cache line. */
const uint8_t CyFxUsbDscrAlignBuffer[32] __attribute__ ((aligned (32)));

/* [ ] */

#endif
