#include "cyu3system.h"
#include "cyu3os.h"
#include "cyu3dma.h"
#include "cyu3error.h"
#include "cyfxuvcinmem.h"
#include "version.h"
#include "cyfxuvcdscr.h"
#include "cyu3usb.h"
#include "cyu3uart.h"
#include "cyu3utils.h"
#include "cyu3i2c.h"
#include <uart_regs.h>
#include <stdint.h>
#include <stdarg.h>
#include <stddef.h>
#include <cyu3gpio.h>
#include <cyu3pib.h>
#include <cyu3socket.h>
#include <sys/types.h>
#include "fx3_uvc.cydsn/cyfxgpif2config.h"

/* Setup data field : Request */
#define CY_U3P_USB_REQUEST_MASK                       (0x0000FF00)
#define CY_U3P_USB_REQUEST_POS                        (8)

CyU3PDmaChannel   chHandleUSBSerial_IN;          /* DMA AUTO (USB TO UART) channel handle.*/
CyU3PDmaChannel   chHandleUSBSerial_OUT;          /* DMA AUTO_SIG(UART TO USB) channel handle.*/
CyU3PDmaChannel glI2cTxHandle;   /* I2C Tx channel handle */
CyU3PDmaChannel glI2cRxHandle;   /* I2C Rx channel handle */
CyBool_t          glIsAppUARTlnActive = CyFalse;    /* Whether the application is active or not. */
CyU3PUartConfig_t glUartConfig = {0};           /* Current UART configuration. */
volatile uint16_t glPktsPending = 0;            /* Number of packets that have been committed since last check. */
static CyU3PEvent    glFxUVCEvent;

volatile static uint32_t glFrameCount = 0;              /* Number of video frames transferred so far. */
volatile static uint32_t glDmaDone = 1;
static uint8_t glDmaResetFlag = CY_FX_UVC_DMA_RESET_EVENT_NOT_ACTIVE;
static int glDmaResetResult = 0;
static uint8_t glUvcVcErrorCode = CY_FX_UVC_VC_ERROR_CODE_NO_ERROR;
uint16_t glI2cPageSize = 0x40;

static char MARKER[] __attribute__((used)) = "AM2_F";
const uint8_t glFirmwareID[32] __attribute__ ((aligned (32))) = { 'F', 'X', '3', 'P', 'R', 'O', 'G', '\0' };


#define SET_LINE_CODING        0x20
#define GET_LINE_CODING        0x21
#define SET_CONTROL_LINE_STATE 0x22

static uint8_t cmdBuff[7] = {0,};

static CyU3PThread uvcAppThread;           /* Thread structure */
static CyU3PThread USBUARTAppThread;
static CyU3PThread   uvcAppEP0Thread;
static CyU3PThread   UartParserThread;
volatile uint16_t glUartPacketsPending = 0;
/* UVC Header */
uint8_t glUVCHeader[CY_FX_UVC_MAX_HEADER] =
{
    0x0C,                           /* Header Length */
    0x8C,                           /* Bit field header field */
    0x00,0x00,0x00,0x00,            /* Presentation time stamp field */
    0x00,0x00,0x00,0x00,0x00,0x00   /* Source clock reference field */
};

/* Video Probe Commit Control */
uint8_t glCommitCtrl[CY_FX_UVC_MAX_PROBE_SETTING_ALIGNED] __attribute__ ((aligned (32)));

CyU3PDmaMultiChannel          glChHandleUVCStream;           /* DMA Channel Handle  */
static volatile CyBool_t glIsApplnActive = CyFalse;     /* Whether the loopback application is active or not. */
static volatile CyBool_t glIsDevConfigured = CyFalse;   /* Whether SET_CONFIG is complete or not. */
static uint8_t glCommitBufferFailureCount = 0;
uint8_t  bmReqType, bmType, bRequest;                           /* bmReqType and bRequest fields. */
uint16_t wValue, wIndex, wLength;
uint8_t gl_commit_count;
uint8_t gl_commit_count_limit_reached = 0;
CyU3PUSBSpeed_t usbSpeed = CY_U3P_NOT_CONNECTED;
static uint8_t glEp0Buffer[2048];

/* UVC Probe Control Settings for a USB 3.0 connection. */
uint8_t glProbeCtrl[CY_FX_UVC_MAX_PROBE_SETTING] = {
    0x00, 0x00,                 /* bmHint : no hit */
    0x01,                       /* Use 1st Video format index */
    0x01,                       /* Use 1st Video frame index */
    IMAGE_FPS,     				/* Desired frame interval in the unit of 100ns: 30 fps */
    0x00, 0x00,                 /* Key frame rate in key frame/video frame units: only applicable
                                   to video streaming with adjustable compression parameters */
    0x00, 0x00,                 /* PFrame rate in PFrame / key frame units: only applicable to
                                   video streaming with adjustable compression parameters */
    0x00, 0x00,                 /* Compression quality control: only applicable to video streaming
                                   with adjustable compression parameters */
    0x00, 0x00,                 /* Window size for average bit rate: only applicable to video
                                   streaming with adjustable compression parameters */
    0x00, 0x00,                 /* Internal video streaming i/f latency in ms */
    0x80,0xc4,0x0b,0x00,     	/* Max video frame size in bytes */
    0x00, 0x40, 0x00, 0x00,      /* No. of bytes device can rx in single payload = 16 KB */

    /* UVC 1.1 Probe Control has additional fields from UVC 1.0 */
    //0x00, 0x60, 0xE3, 0x16,             /* Device Clock */
    0x00, 0x84, 0xd7, 0x17,             /* Device Clock */
    0x00,                               /* Framing Information - Ignored for uncompressed format*/
    0x00,                               /* Preferred payload format version */
    0x00,                               /* Minimum payload format version */
    0x00                                /* Maximum payload format version */

};

/* UVC Probe Control Setting for a USB 2.0 connection. */
uint8_t glProbeCtrl20[CY_FX_UVC_MAX_PROBE_SETTING] = {
    0x00, 0x00,                 /* bmHint : no hit */
    0x01,                       /* Use 1st Video format index */
    0x01,                       /* Use 1st Video frame index */
    IMAGE_FPS,     /* Desired frame interval in the unit of 100ns: 15 fps */
    0x00, 0x00,                 /* Key frame rate in key frame/video frame units: only applicable
                                   to video streaming with adjustable compression parameters */
    0x00, 0x00,                 /* PFrame rate in PFrame / key frame units: only applicable to
                                   video streaming with adjustable compression parameters */
    0x00, 0x00,                 /* Compression quality control: only applicable to video streaming
                                   with adjustable compression parameters */
    0x00, 0x00,                 /* Window size for average bit rate: only applicable to video
                                   streaming with adjustable compression parameters */
    0x00, 0x00,                 /* Internal video streaming i/f latency in ms */
    0x80,0xc4,0x0b,0x00,     /* Max video frame size in bytes */
    0x00, 0x40, 0x00, 0x00,      /* No. of bytes device can rx in single payload = 16 KB */


    /* UVC 1.1 Probe Control has additional fields from UVC 1.0 */
    0x00, 0x84, 0xd7, 0x17,             /* Device Clock */
    0x00,                               /* Framing Information - Ignored for uncompressed format*/
    0x00,                               /* Preferred payload format version */
    0x00,                               /* Minimum payload format version */
    0x00                                /* Maximum payload format version */

};

void CyFxUSBUARTAppStop();
void CyFxUSBUARTAppStart();

#ifdef DEBUG_UART
void *_sbrk ( uint32_t delta ) {
	static char end;
	static char *heap_end;
	char *prev_heap_end;

	if (heap_end == 0) {
		heap_end = &end;
	}

	prev_heap_end = heap_end;
	heap_end += delta;
	return (void *) prev_heap_end;
}

void uart_printf(char *fmt, ...) {
	va_list args;
    va_start(args, fmt);
    int size = vsnprintf(NULL, 0, fmt, args);
    va_end(args);


    va_start(args, fmt);
    char * result = (char *)CyU3PMemAlloc(size+1);
    vsprintf(result, fmt, args);
    va_end(args);

    CyU3PReturnStatus_t status;
    CyU3PUartTransmitBytes (result, size, &status);

    CyU3PMemFree(result);
}
#else
void uart_printf(char *fmt, ...) {


}
#endif

/* Application error handler */
void
CyFxAppErrorHandler(
		int line,
        CyU3PReturnStatus_t apiRetStatus    /* API return status */
        )
{
    /* Application failed with the error code apiRetStatus */

    /* Add custom debug or recovery actions here */
	/* Loop indefinitely */
	uart_printf( "CyFxAppErrorHandler. Error=%d at line %d\n", apiRetStatus, line);

    //for (;;) CyU3PThreadSleep (1000);
}



void CyFxUART2USBDmaCallback(
        CyU3PDmaChannel   *chHandle, /* Handle to the DMA channel. */
        CyU3PDmaCbType_t   type,     /* Callback type.             */
        CyU3PDmaCBInput_t *input){
	if (type == CY_U3P_DMA_CB_PROD_EVENT)
	{

		CyU3PDmaChannelCommitBuffer (&chHandleUSBSerial_OUT, input->buffer_p.count, 0);
		glUartPacketsPending++;
		//CyU3PUartTransmitBytes (input->buffer_p.buffer, input->buffer_p.count, &status);
	}
}


CyU3PReturnStatus_t
CyFxFlashProgI2cInit (uint16_t pageLen)
{
    CyU3PI2cConfig_t i2cConfig;
    CyU3PDmaChannelConfig_t dmaConfig;
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;

    /* Initialize and configure the I2C master module. */
    status = CyU3PI2cInit ();
    if (status != CY_U3P_SUCCESS)
    {
        return status;
    }

    /* Start the I2C master block. The bit rate is set at 100KHz.
     * The data transfer is done via DMA. */
    CyU3PMemSet ((uint8_t *)&i2cConfig, 0, sizeof(i2cConfig));
    i2cConfig.bitRate    = 100000;
    i2cConfig.busTimeout = 0xFFFFFFFF;
    i2cConfig.dmaTimeout = 0xFFFF;
    i2cConfig.isDma      = CyTrue;

    status = CyU3PI2cSetConfig (&i2cConfig, NULL);
    if (status != CY_U3P_SUCCESS)
    {
        return status;
    }

    /* Now create the DMA channels required for read and write. */
    CyU3PMemSet ((uint8_t *)&dmaConfig, 0, sizeof(dmaConfig));
    dmaConfig.size           = pageLen;
    /* No buffers need to be allocated as this will be used
     * only in override mode. */
    dmaConfig.count          = 0;
    dmaConfig.prodAvailCount = 0;
    dmaConfig.dmaMode        = CY_U3P_DMA_MODE_BYTE;
    dmaConfig.prodHeader     = 0;
    dmaConfig.prodFooter     = 0;
    dmaConfig.consHeader     = 0;
    dmaConfig.notification   = 0;
    dmaConfig.cb             = NULL;

    /* Create a channel to write to the EEPROM. */
    dmaConfig.prodSckId = CY_U3P_CPU_SOCKET_PROD;
    dmaConfig.consSckId = CY_U3P_LPP_SOCKET_I2C_CONS;
    status = CyU3PDmaChannelCreate (&glI2cTxHandle,
            CY_U3P_DMA_TYPE_MANUAL_OUT, &dmaConfig);
    if (status != CY_U3P_SUCCESS)
    {
    	uart_printf("I2C TX CyU3PDmaChannelCreate  error: %x\n", status);
        return status;
    }

    /* Create a channel to read from the EEPROM. */
    dmaConfig.prodSckId = CY_U3P_LPP_SOCKET_I2C_PROD;
    dmaConfig.consSckId = CY_U3P_CPU_SOCKET_CONS;
    status = CyU3PDmaChannelCreate (&glI2cRxHandle,
            CY_U3P_DMA_TYPE_MANUAL_IN, &dmaConfig);

    if (status == CY_U3P_SUCCESS)
    {
        glI2cPageSize = pageLen;
    } else {
    	uart_printf("I2C RX CyU3PDmaChannelCreate error: %x\n", status);
    }

    return status;
}


/* I2C read / write for programmer application. */
CyU3PReturnStatus_t
CyFxFlashProgI2cTransfer (
        uint16_t  byteAddress,
        uint8_t   devAddr,
        uint16_t  byteCount,
        uint8_t  *buffer,
        CyBool_t  isRead)
{
    CyU3PDmaBuffer_t buf_p;
    CyU3PI2cPreamble_t preamble;
    uint16_t pageCount = (byteCount / glI2cPageSize);
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;

    if (byteCount == 0)
    {
        return CY_U3P_SUCCESS;
    }

    if ((byteCount % glI2cPageSize) != 0)
    {
        pageCount ++;
    }

    uart_printf("I2C access - dev: 0x%x, address: 0x%x, size: 0x%x, pages: 0x%x.\r\n", devAddr, byteAddress, byteCount, pageCount);

    /* Update the buffer address and status. */
    buf_p.buffer = buffer;
    buf_p.status = 0;

    while (pageCount != 0)
    {
        if (isRead)
        {
            /* Update the preamble information. */
            preamble.length    = 4;
            preamble.buffer[0] = devAddr;
            preamble.buffer[1] = (uint8_t)(byteAddress >> 8);
            preamble.buffer[2] = (uint8_t)(byteAddress & 0xFF);
            preamble.buffer[3] = (devAddr | 0x01);
            preamble.ctrlMask  = 0x0004;

            buf_p.size = glI2cPageSize;
            buf_p.count = glI2cPageSize;

            status = CyU3PI2cSendCommand (&preamble, glI2cPageSize, isRead);
            if (status != CY_U3P_SUCCESS)
            {
            	uart_printf("CyU3PI2cSendCommand error\n");
                return status;
            }
            status = CyU3PDmaChannelSetupRecvBuffer (&glI2cRxHandle, &buf_p);
            if (status != CY_U3P_SUCCESS)
            {
            	uart_printf("CyU3PDmaChannelSetupRecvBuffer error\n");
                return status;
            }
            status = CyU3PDmaChannelWaitForCompletion(&glI2cRxHandle,
                    CY_FX_FLASH_PROG_TIMEOUT);
            if (status != CY_U3P_SUCCESS)
            {
            	uart_printf("CyU3PDmaChannelWaitForCompletion error\n");
                return status;
            }
        }
        else /* Write */
        {
            /* Update the preamble information. */
            preamble.length    = 3;
            preamble.buffer[0] = devAddr;
            preamble.buffer[1] = (uint8_t)(byteAddress >> 8);
            preamble.buffer[2] = (uint8_t)(byteAddress & 0xFF);
            preamble.ctrlMask  = 0x0000;

            buf_p.size = glI2cPageSize;
            buf_p.count = glI2cPageSize;

            status = CyU3PDmaChannelSetupSendBuffer (&glI2cTxHandle,
                    &buf_p);
            if (status != CY_U3P_SUCCESS)
            {
            	uart_printf("CyU3PDmaChannelSetupSendBuffer error: %x\n", status);
                return status;
            }
            status = CyU3PI2cSendCommand (&preamble, glI2cPageSize, isRead);
            if (status != CY_U3P_SUCCESS)
            {
            	uart_printf("CyU3PI2cSendCommand error: %x\n", status);
                return status;
            }
            status = CyU3PDmaChannelWaitForCompletion(&glI2cTxHandle,
                    CY_FX_FLASH_PROG_TIMEOUT);
            if (status != CY_U3P_SUCCESS)
            {
            	uart_printf("CyU3PDmaChannelWaitForCompletion error: %x\n", status);
                return status;
            }
        }

        /* Update the parameters */
        byteAddress  += glI2cPageSize;
        buf_p.buffer += glI2cPageSize;
        pageCount --;

        /* Need a delay between write operations. */
        CyU3PThreadSleep (10);
    }

    return CY_U3P_SUCCESS;
}
/* This function initializes the debug module for the UVC application */
void
CyFxUArtInit (void)
{
    CyU3PUartConfig_t uartConfig;
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;

    /* Initialize the UART for printing debug messages */
    apiRetStatus = CyU3PUartInit();
    if (apiRetStatus != CY_U3P_SUCCESS)
    {

    	CyFxAppErrorHandler(__LINE__,apiRetStatus);
    }

    /* Set UART Configuration */
    CyU3PMemSet ((uint8_t *)&uartConfig, 0, sizeof (uartConfig));
    uartConfig.baudRate = CY_U3P_UART_BAUDRATE_230400;
    uartConfig.stopBit = CY_U3P_UART_ONE_STOP_BIT;
    uartConfig.parity = CY_U3P_UART_NO_PARITY;
    uartConfig.txEnable = CyTrue;
    uartConfig.rxEnable = CyTrue;
    uartConfig.flowCtrl = CyFalse;
#ifdef DEBUG_UART
    uartConfig.isDma = CyFalse;
#else
    uartConfig.isDma = CyTrue;//CyFalse;
#endif


    /* Set the UART configuration */
    //apiRetStatus = CyU3PUartSetConfig (&uartConfig,CyFxUartCallback);
    apiRetStatus = CyU3PUartSetConfig (&uartConfig,NULL);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyFxAppErrorHandler(__LINE__,apiRetStatus);
    }

    /* Set the UART transfer
    //apiRetStatus = CyU3PUartTxSetBlockXfer (0xFFFFFFFF);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyFxAppErrorHandler(__LINE__,apiRetStatus);
    }

    // Initialize the debug application
    apiRetStatus = CyU3PDebugInit (CY_U3P_LPP_SOCKET_UART_CONS, 8);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyFxAppErrorHandler(__LINE__,apiRetStatus);
    }*/
    //apiRetStatus = CyU3PDebugInit (CY_U3P_LPP_SOCKET_UART_CONS, 8);

    CyU3PDebugPreamble (CyFalse);
}



void
CyFxUSBUARTAppStart(
        void )
{
	   uint16_t size = 0;
	    CyU3PEpConfig_t epCfg;
	    CyU3PDmaChannelConfig_t dmaCfg;
	    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;
	    CyU3PUSBSpeed_t usbSpeed = CyU3PUsbGetSpeed();

	    /* Based on the Bus speed configure the endpoint packet size */
	    switch (usbSpeed)
	    {
	        case CY_U3P_FULL_SPEED:
	            size = 16;
	            break;

	        case CY_U3P_HIGH_SPEED:
	            size = 512;
	            break;

	        case  CY_U3P_SUPER_SPEED:
	            /* Turning low power mode off to avoid USB transfer delays. */
	            //CyU3PUsbLPMDisable ();
	            size = 1024;
	            break;

	        default:
	            CyFxAppErrorHandler (__LINE__, CY_U3P_ERROR_FAILURE);
	            break;
	    }

	    CyU3PMemSet ((uint8_t *)&epCfg, 0, sizeof (epCfg));
	    epCfg.enable = CyTrue;
	    epCfg.epType = CY_U3P_USB_EP_BULK;
	    epCfg.burstLen = 1;
	    epCfg.streams = 0;
	    epCfg.pcktSize = size;

	    /* Producer endpoint configuration */
	    apiRetStatus = CyU3PSetEpConfig(CY_FX_EP_PRODUCER , &epCfg);
	    if (apiRetStatus != CY_U3P_SUCCESS)
	    {
	        CyFxAppErrorHandler (__LINE__, apiRetStatus);
	    }

	    /* Consumer endpoint configuration */
	    apiRetStatus = CyU3PSetEpConfig(CY_FX_EP_CONSUMER, &epCfg);
	    if (apiRetStatus != CY_U3P_SUCCESS)
	    {
	        CyFxAppErrorHandler (__LINE__, apiRetStatus);
	    }

	    /* Interrupt endpoint configuration */
	    epCfg.epType = CY_U3P_USB_EP_INTR;
	    epCfg.pcktSize = 64;
	    epCfg.isoPkts = 1;

	    apiRetStatus = CyU3PSetEpConfig(CY_FX_EP_INTERRUPT, &epCfg);
	    if (apiRetStatus != CY_U3P_SUCCESS)
	    {
	        CyFxAppErrorHandler(__LINE__, apiRetStatus);
	    }


	// From USB3 to UART

    /* Create channel between usb producer socket and uart consumer socket */
    dmaCfg.size = size;
    dmaCfg.count = CY_FX_USBUART_DMA_BUF_COUNT;
    dmaCfg.prodSckId = CY_FX_EP_PRODUCER1_SOCKET;
    dmaCfg.consSckId = CY_FX_EP_CONSUMER1_SOCKET;//CY_U3P_CPU_SOCKET_CONS;
    dmaCfg.dmaMode = CY_U3P_DMA_MODE_BYTE;
    dmaCfg.notification = 0;//CY_U3P_DMA_CB_PROD_EVENT;
    dmaCfg.cb = NULL;//CyFxUSB2UARTDmaCallback;
    dmaCfg.prodHeader = 0;
    dmaCfg.prodFooter = 0;
    dmaCfg.consHeader = 0;
    dmaCfg.prodAvailCount = 0;

    //apiRetStatus = CyU3PDmaChannelCreate (&chHandleUSBSerial_IN,CY_U3P_DMA_TYPE_MANUAL_IN, &dmaCfg);
    apiRetStatus = CyU3PDmaChannelCreate (&chHandleUSBSerial_IN,CY_U3P_DMA_TYPE_AUTO, &dmaCfg);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyFxAppErrorHandler(__LINE__,apiRetStatus);
    }
    // From UART to USB3

#ifndef DEBUG_UART
    /* Create a DMA_MANUAL channel between uart producer socket and usb consumer socket */
    /* Use a smaller buffer size (32 bytes) to ensure that packets get filled in a short time. */
    dmaCfg.size         = 16;
    //dmaCfg.count 		= CY_FX_USBUART_DMA_BUF_COUNT;//1;
    dmaCfg.prodSckId    = CY_FX_EP_PRODUCER2_SOCKET;//CY_U3P_CPU_SOCKET_PROD;
    dmaCfg.consSckId    = CY_FX_EP_CONSUMER2_SOCKET;
    dmaCfg.notification = CY_U3P_DMA_CB_PROD_EVENT;
    dmaCfg.cb           = CyFxUART2USBDmaCallback;

    //apiRetStatus = CyU3PDmaChannelCreate (&chHandleUSBSerial_OUT, CY_U3P_DMA_TYPE_MANUAL_OUT, &dmaCfg);
    apiRetStatus = CyU3PDmaChannelCreate (&chHandleUSBSerial_OUT, CY_U3P_DMA_TYPE_MANUAL, &dmaCfg);

    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyFxAppErrorHandler(__LINE__,apiRetStatus);
    }
#endif
    /* Set DMA Channel transfer size */
    apiRetStatus = CyU3PDmaChannelSetXfer (&chHandleUSBSerial_IN,0);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyFxAppErrorHandler(__LINE__,apiRetStatus);
    }
#ifndef DEBUG_UART
    apiRetStatus = CyU3PDmaChannelSetXfer (&chHandleUSBSerial_OUT, 0);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyFxAppErrorHandler(__LINE__,apiRetStatus);
    }
#endif
    /* Update the status flag. */
    glIsAppUARTlnActive = CyTrue;
}

void
CyFxUSBUARTAppStop (
        void)
{
    CyU3PEpConfig_t epCfg;
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;

    /* Update the flag. */
    glIsAppUARTlnActive = CyFalse;

    /* Flush the endpoint memory */
    CyU3PUsbFlushEp(CY_FX_EP_PRODUCER);
    CyU3PUsbFlushEp(CY_FX_EP_CONSUMER);
    CyU3PUsbFlushEp(CY_FX_EP_INTERRUPT);

    /* Destroy the channel */
    CyU3PDmaChannelDestroy (&chHandleUSBSerial_IN);
    CyU3PDmaChannelDestroy (&chHandleUSBSerial_OUT);

    /* Disable endpoints. */
    CyU3PMemSet ((uint8_t *)&epCfg, 0, sizeof (epCfg));
    epCfg.enable = CyFalse;

    /* Producer endpoint configuration. */
    apiRetStatus = CyU3PSetEpConfig(CY_FX_EP_PRODUCER, &epCfg);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyFxAppErrorHandler(__LINE__,apiRetStatus);
    }

    /* Consumer endpoint configuration. */
    apiRetStatus = CyU3PSetEpConfig(CY_FX_EP_CONSUMER, &epCfg);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyFxAppErrorHandler(__LINE__,apiRetStatus);
    }

    /* Interrupt endpoint configuration. */
    apiRetStatus = CyU3PSetEpConfig(CY_FX_EP_INTERRUPT, &epCfg);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyFxAppErrorHandler(__LINE__,apiRetStatus);
    }
}

/* UVC header addition function */
static void
CyFxUVCAddHeader (
        uint8_t *buffer_p, /* Buffer pointer */
        uint8_t frameInd   /* EOF or normal frame indication */
    )
{
    /* Copy header to buffer */
    CyU3PMemCopy (buffer_p, (uint8_t *)glUVCHeader, CY_FX_UVC_MAX_HEADER);

    /* Check if last packet of the frame. */
    if (frameInd == CY_FX_UVC_HEADER_EOF)
    {
        /* Modify UVC header to toggle Frame ID */
        glUVCHeader[1] ^= CY_FX_UVC_HEADER_FRAME_ID;

        /* Indicate End of Frame in the buffer */
        buffer_p[1] |=  CY_FX_UVC_HEADER_EOF;
    }
}

void
CyFxUvcApplnDmaCallback (
        CyU3PDmaMultiChannel *chHandle,
        CyU3PDmaCbType_t      type,
        CyU3PDmaCBInput_t    *input
        )
{
    CyU3PDmaBuffer_t    dmaBuffer;
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
    CyBool_t not_skip = CyTrue;

    if (type == CY_U3P_DMA_CB_PROD_EVENT)
    {
    	if (gl_commit_count == (2* CY_FX_UVC_STREAM_BUF_COUNT))
    	{
			gl_commit_count_limit_reached++;
			glDmaResetFlag = CY_FX_UVC_DMA_RESET_COMMIT_BUFFER_FAILURE;
			CyU3PEventSet(&glFxUVCEvent, CY_FX_UVC_DMA_RESET_EVENT, CYU3P_EVENT_OR);
			not_skip = CyFalse;
    	}
    	if (not_skip)
    	{
    		status = CyU3PDmaMultiChannelGetBuffer (chHandle, &dmaBuffer, CYU3P_NO_WAIT);
    		while (status == CY_U3P_SUCCESS)
    		{
    			/* Add Headers*/
    			if (dmaBuffer.count == CY_FX_UVC_BUF_FULL_SIZE)
    			{
    				/* A full buffer indicates there is more data to go in this video frame. */
    				CyFxUVCAddHeader (dmaBuffer.buffer - CY_FX_UVC_MAX_HEADER, CY_FX_UVC_HEADER_FRAME);
    			}
    			else
    			{
    				/* A partially filled buffer indicates the end of the ongoing video frame. */
    				CyFxUVCAddHeader (dmaBuffer.buffer - CY_FX_UVC_MAX_HEADER, CY_FX_UVC_HEADER_EOF);

    				glFrameCount++;
    				glDmaDone = 0;
    			}

    			/* Commit Buffer to USB*/
    			status = CyU3PDmaMultiChannelCommitBuffer (chHandle, (dmaBuffer.count + CY_FX_UVC_MAX_HEADER), 0);
    			if (status == CY_U3P_SUCCESS)
    			{
    				glDmaDone++;
    				gl_commit_count++;
    			}
    			else
    			{
    				glDmaResetResult = status;
    				if(glDmaResetFlag == CY_FX_UVC_DMA_RESET_EVENT_NOT_ACTIVE)
    				{
    					glDmaResetFlag = CY_FX_UVC_DMA_RESET_COMMIT_BUFFER_FAILURE;
    					CyU3PEventSet(&glFxUVCEvent, CY_FX_UVC_DMA_RESET_EVENT, CYU3P_EVENT_OR);
    				}
    				break;
    			}
    			if (gl_commit_count == (2* CY_FX_UVC_STREAM_BUF_COUNT)) {
    				gl_commit_count_limit_reached++;
    				glDmaResetFlag = CY_FX_UVC_DMA_RESET_COMMIT_BUFFER_FAILURE;
    				CyU3PEventSet(&glFxUVCEvent, CY_FX_UVC_DMA_RESET_EVENT, CYU3P_EVENT_OR);
    				break;
    			}

    			/* Check if any more buffers are ready to go, and commit them here. */

            		status = CyU3PDmaMultiChannelGetBuffer (chHandle, &dmaBuffer, CYU3P_NO_WAIT);
    		}
    	}
    }
    else if (type == CY_U3P_DMA_CB_CONS_EVENT)
    {
    	gl_commit_count--;
        //streamingStarted = CyTrue;
        //glCommitBufferFailureCount = 0;        /* Reset the counter after data is consumed by USB */
    }
}

/* This function starts the video streaming application. It is called
 * when there is a SET_INTERFACE event for alternate interface 1. */
CyU3PReturnStatus_t
CyFxUVCApplnStart (void)
{
	uart_printf( " *** CyFxUVCApplnStart ***\n");
    CyU3PEpConfig_t endPointConfig;
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;
    glDmaDone    = 1;
    glFrameCount = 0;
    glUVCHeader[1] &= ~CY_FX_UVC_HEADER_FRAME_ID;

    /* Video streaming endpoint configuration */


    CyU3PUsbLPMDisable ();
    if (CyU3PUsbGetSpeed () == CY_U3P_SUPER_SPEED)
    {
    	CyU3PUsbSetLinkPowerState (CyU3PUsbLPM_U0);
    	CyU3PBusyWait (200);
    }
    else
    {
    	CyU3PUsb2Resume ();
    }

    CyU3PUsbSetEpNak (CY_FX_EP_BULK_VIDEO, CyTrue);
    CyU3PBusyWait (125);

    CyU3PUsbFlushEp (CY_FX_EP_BULK_VIDEO);
    CyU3PDmaMultiChannelReset (&glChHandleUVCStream);
    apiRetStatus = CyU3PDmaMultiChannelSetXfer (&glChHandleUVCStream, 0, 0);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
    	uart_printf( "DMA Channel Set Transfer Failed, Error Code = %d\r\n", apiRetStatus);
    	CyFxAppErrorHandler(__LINE__,apiRetStatus);
    }
    CyU3PUsbSetEpNak (CY_FX_EP_BULK_VIDEO, CyFalse);
    CyU3PBusyWait (125);

    apiRetStatus = CyU3PGpifSMSwitch(CY_FX_UVC_INVALID_GPIF_STATE, START_SCK0,CY_FX_UVC_INVALID_GPIF_STATE, ALPHA_START_SCK0, 2);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
            /* Error Handling */
    	uart_printf( "Switching GPIF state machine failed, Error Code = %d\r\n", apiRetStatus);
    	CyFxAppErrorHandler(__LINE__,apiRetStatus);
	}


    gl_commit_count_limit_reached = 0;
    return CY_U3P_SUCCESS;
}

/* This function stops the video streaming. It is called from the USB event
 * handler, when there is a reset / disconnect or SET_INTERFACE for alternate
 * interface 0. */
void
CyFxUVCApplnStop (void)
{
    CyU3PGpifDisable (CyFalse);
    glDmaResetFlag = CY_FX_UVC_DMA_RESET_EVENT_NOT_ACTIVE;

        /* Place the EP in NAK mode before cleaning up the pipe. */
    CyU3PUsbSetEpNak (CY_FX_EP_BULK_VIDEO, CyTrue);
    CyU3PBusyWait (125);


    /* Reset and flush the endpoint pipe. */
    CyU3PDmaMultiChannelReset (&glChHandleUVCStream);
    CyU3PUsbFlushEp (CY_FX_EP_BULK_VIDEO);
    CyU3PUsbSetEpNak (CY_FX_EP_BULK_VIDEO, CyFalse);
    CyU3PBusyWait (125);

    gl_commit_count = 0;

    /* Allow USB low power link transitions at this stage. */
    //CyU3PUsbLPMEnable ();
    uart_printf( " *** CyFxUVCApplnStop ***\n");
}

static void
CyFxUVCApplnAbortHandler (
        void)
{
    /* Set Video Stream Abort Event */
    CyU3PEventSet (&glFxUVCEvent, CY_FX_UVC_STREAM_ABORT_EVENT, CYU3P_EVENT_OR);
}

/* This is the Callback function to handle the USB Events */
static void
CyFxUVCApplnUSBEventCB (
    CyU3PUsbEventType_t evtype, /* Event type */
    uint16_t            evdata  /* Event data */
    )
{
	//uart_printf( "CyFxUVCApplnUSBEventCB: %u, %u\r\n",evtype,evdata);
    switch (evtype)
    {
        case CY_U3P_USB_EVENT_SETCONF:
            if (evdata != 0)
                glIsDevConfigured = CyTrue;
            else
                glIsDevConfigured = CyFalse;

            if (glIsAppUARTlnActive)
			{
				CyFxUSBUARTAppStop();
			}
			/* Start the loop back function. */
//#ifndef DEBUG_UART
			CyFxUSBUARTAppStart();
//#endif
        case CY_U3P_USB_EVENT_SETINTF:
             /* Stop the application before re-starting. */
             if (glIsApplnActive)
             {
            	 CyFxUVCApplnAbortHandler();
             }

             CyU3PUsbLPMDisable ();
             break;

        case CY_U3P_USB_EVENT_CONNECT:
        case CY_U3P_USB_EVENT_RESET:
        case CY_U3P_USB_EVENT_DISCONNECT:
            /* Stop the video streamer application. */
            if (glIsApplnActive)
            {
                CyFxUVCApplnStop ();
            }
            glIsDevConfigured = CyFalse;

            CyU3PUsbLPMEnable ();
            if (glIsAppUARTlnActive)
			{
				CyFxUSBUARTAppStop ();
			}

        	CyU3PDmaChannelReset (&glI2cTxHandle);
        	CyU3PDmaChannelReset (&glI2cRxHandle);
            break;
        case CY_U3P_USB_EVENT_SUSPEND:
            //uart_printf( "UsbEventCB: SUSPEND encountered...\r\n");
            /* Set USB suspend Event */
            CyU3PEventSet (&glFxUVCEvent, CY_FX_USB_SUSPEND_EVENT_HANDLER, CYU3P_EVENT_OR);
            break;
        default:
            break;
    }
}

static CyBool_t UVCRequestHandler(CyBool_t isHandled_prev, uint8_t  bmReqType, uint8_t bmType, uint8_t bRequest, uint16_t wValue, uint16_t wIndex, uint16_t wLength) {
	CyBool_t isHandled = isHandled_prev;

	switch (bmReqType) {
						case CY_FX_USB_UVC_GET_REQ_TYPE:
						case CY_FX_USB_UVC_SET_REQ_TYPE:
							switch (wIndex & 0xFF) {
								case CY_FX_UVC_CONTROL_INTERFACE:
									//uart_printf("\t\tCY_FX_UVC_CONTROL_INTERFACE\n");
										CyU3PEventSet (&glFxUVCEvent, CY_FX_UVC_VIDEO_CONTROL_REQUEST_EVENT,CYU3P_EVENT_OR);

										isHandled = CyTrue;
									break;
								case CY_FX_UVC_STREAM_INTERFACE:
									//uart_printf("\t\tCY_FX_UVC_STREAM_INTERFACE\n");
										CyU3PEventSet (&glFxUVCEvent, CY_FX_UVC_VIDEO_STREAM_REQUEST_EVENT, CYU3P_EVENT_OR);

										isHandled = CyTrue;
									break;
								default:
									isHandled = CyFalse;
									break;
							}
		                break;
						case CY_FX_USB_SET_INTF_REQ_TYPE:
							if (bRequest == CY_FX_USB_SET_INTERFACE_REQ) {
								//uart_printf("\tCY_FX_USB_SET_INTERFACE_REQ\n");
								if ((wIndex == CY_FX_UVC_STREAM_INTERFACE) && (wValue == 0)) {
									CyU3PUsbStall (CY_FX_EP_BULK_VIDEO, CyFalse, CyTrue);
									CyFxUVCApplnAbortHandler ();
									isHandled = CyTrue;
									CyU3PUsbAckSetup ();
								}
							} else
							if ((bRequest == CY_U3P_USB_SC_SET_FEATURE) || (bRequest == CY_U3P_USB_SC_CLEAR_FEATURE)) {
							//uart_printf("\t\tCY_U3P_USB_SC_SET_FEATURE / CY_U3P_USB_SC_CLEAR_FEATURE\n");
								if (glIsDevConfigured) {
									//uart_printf("\t\t\tglIsDevConfigured\n");
									isHandled = CyTrue;
									CyU3PUsbAckSetup ();
								}
							}

							break;
						case CY_U3P_USB_TARGET_ENDPT:
							//uart_printf("\t\tCY_U3P_USB_TARGET_ENDPT\n");
							if ((bRequest == CY_U3P_USB_SC_CLEAR_FEATURE) && (wIndex == CY_FX_EP_BULK_VIDEO)) {
								//uart_printf("\t\t\tCY_U3P_USB_SC_CLEAR_FEATURE\n");
								CyU3PUsbStall (CY_FX_EP_BULK_VIDEO, CyFalse, CyTrue);
								CyFxUVCApplnAbortHandler();
								isHandled = CyTrue;
								CyU3PUsbAckSetup ();
							}
							//uart_printf("\t\tCY_U3P_USB_TARGET_ENDPT\n");
							break;

						default:
							isHandled = CyFalse;
							break;
					}
	return isHandled;
}

/* Callback to handle the USB Setup Requests and UVC Class events */
static CyBool_t
CyFxUVCApplnUSBSetupCB (
        uint32_t setupdat0, /* SETUP Data 0 */
        uint32_t setupdat1  /* SETUP Data 1 */
    )
{
    uint16_t readCount = 0;
    uint8_t config_data[7];

    CyBool_t isHandled = CyFalse;
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;
    CyU3PUartConfig_t uartConfig;
    uint8_t  i2cAddr = 0;
    uint32_t *addr = NULL;
    int32_t offset = 0;

    bmReqType = (uint8_t)(setupdat0 & CY_U3P_USB_REQUEST_TYPE_MASK);
    bmType    = (bmReqType & CY_U3P_USB_TYPE_MASK);
    bRequest  = (uint8_t)((setupdat0 & CY_U3P_USB_REQUEST_MASK) >> CY_U3P_USB_REQUEST_POS);
    wValue    = (uint16_t)((setupdat0 & CY_U3P_USB_VALUE_MASK) >> CY_U3P_USB_VALUE_POS);
    wIndex    = (uint16_t)(setupdat1 & CY_U3P_USB_INDEX_MASK);
    wLength   = (uint16_t)((setupdat1 & CY_U3P_USB_LENGTH_MASK) >> CY_U3P_USB_LENGTH_POS);
    uint8_t bTarget  = (bmReqType & CY_U3P_USB_TARGET_MASK);

    //uart_printf( "CyFxUVCApplnUSBSetupCB: [%08x %08x]\tbmReqType:%x, \tbmType: %x, \tbRequest:%x, \tbTarget:%x, \tValue:%x, \tLength:%x, \tIndex:%x\r\n", setupdat0, setupdat1, bmReqType, bmType, bRequest,bTarget, wValue,wLength, wIndex);

    if (bmType == CY_U3P_USB_STANDARD_RQT)
        {
            /* Handle SET_FEATURE(FUNCTION_SUSPEND) and CLEAR_FEATURE(FUNCTION_SUSPEND)
             * requests here. It should be allowed to pass if the device is in configured
             * state and failed otherwise. */
            if ((bTarget == CY_U3P_USB_TARGET_INTF) && ((bRequest == CY_U3P_USB_SC_SET_FEATURE)
                        || (bRequest == CY_U3P_USB_SC_CLEAR_FEATURE)) && (wValue == 0))
            {
            	//uart_printf("Check 1\n");
            	if (glIsApplnActive){
            		//uart_printf("Active\n");
                    CyU3PUsbAckSetup ();
            	}
                else {
                	//uart_printf("not Active\n");
                    CyU3PUsbStall (0, CyTrue, CyFalse);
                }
                isHandled = CyTrue;
            }
        }

    if (bmType == CY_U3P_USB_VENDOR_RQT)
        {
            isHandled = CyTrue;

            switch (bRequest)
            {
                case CY_FX_RQT_ID_CHECK:
                	uart_printf("CY_FX_RQT_ID_CHECK\n");
                    CyU3PUsbSendEP0Data (8, (uint8_t *)glFirmwareID);
                    break;

                case CY_FX_RQT_I2C_EEPROM_WRITE:
                	uart_printf("CY_FX_RQT_I2C_EEPROM_WRITE\n");
                    i2cAddr = 0xA0 | ((wValue & 0x0007) << 1);
                    status  = CyU3PUsbGetEP0Data(wLength, glEp0Buffer, NULL);
                    if (status == CY_U3P_SUCCESS)
                    {
                        CyFxFlashProgI2cTransfer (wIndex, i2cAddr, wLength,
                                glEp0Buffer, CyFalse);
                    }
                    break;

                case CY_FX_RQT_I2C_EEPROM_READ:
                	uart_printf("CY_FX_RQT_I2C_EEPROM_READ\n");
                    i2cAddr = 0xA0 | ((wValue & 0x0007) << 1);
                    CyU3PMemSet (glEp0Buffer, 0, sizeof (glEp0Buffer));
                    status = CyFxFlashProgI2cTransfer (wIndex, i2cAddr, wLength,
                            glEp0Buffer, CyTrue);
                    if (status == CY_U3P_SUCCESS)
                    {
                        status = CyU3PUsbSendEP0Data(wLength, glEp0Buffer);
                        uart_printf("CyFxFlashProgI2cTransfer OK\n");
                    } else
                    	uart_printf("CyFxFlashProgI2cTransfer Failed\n");

                    break;

                case CY_FX_RQT_SYS_MEM_READ:
                	uart_printf("CY_FX_RQT_SYS_MEM_READ\n");
                    addr = (uint32_t *)((wIndex << 16) | wValue);
                    offset = 0;
                    if (wLength)
                    {
                        while (offset < (wLength / 4))
                        {
                            ((uint32_t *)glEp0Buffer)[offset++] = *addr;
                            addr++;
                        }

                        status = CyU3PUsbSendEP0Data (wLength, glEp0Buffer);
                    }
                    else
                    {
                        /* Zero length read. Just ACK.*/
                        CyU3PUsbAckSetup ();
                    }
                    break;

                case CY_FX_RQT_GET_FW_VERSION: {
                    	uart_printf("CY_FX_RQT_GET_FW_VERSION\n");
                    	glEp0Buffer[0] = VERSION_MIN;
                    	glEp0Buffer[1] = VERSION_HOUR;
                    	glEp0Buffer[2] = VERSION_DAY;
                    	glEp0Buffer[3] = VERSION_MONTH;
                    	glEp0Buffer[4] = VERSION_YEAR;
                    	uart_printf("Version: %d.%d.%d - %d:%d\n",glEp0Buffer[2], glEp0Buffer[3],glEp0Buffer[4],glEp0Buffer[1],glEp0Buffer[0]);
                        CyU3PUsbSendEP0Data (5, glEp0Buffer);
                    }
                    break;

                case 0xE0:
                	uart_printf("0xE0\n");
                    CyU3PUsbAckSetup ();
                    CyU3PDeviceReset (CyFalse);
                    break;
                case 0xA0:
                	//uart_printf("Request: ReqType:%x, \tType:%x, \tValue:%x, \tRequest:%x, len=%0x, val=%x, idx=%x\r\n", bmReqType, bmType, bRequest,wLength, wValue, wIndex);

                    switch (wValue) {
                    	case 0:
                    		glEp0Buffer[0] = 0;
                    		CyU3PUsbSendEP0Data (1, glEp0Buffer);
                    		//CyU3PConnectState(CyTrue,CyFalse);
                    		//CyFx3BootJumpToProgramEntry (address);

                    		break;
                    	case 0xE600:
                    		CyU3PUsbStall (0, CyTrue, CyFalse);
                    		 break;
                    	default:
                    		//glEp0Buffer[0] = 0;
                    		//CyU3PUsbSendEP0Data (wLength, glEp0Buffer);
                    		//CyU3PUsbAckSetup ();

                    		i2cAddr = 0xA0 | ((wValue & 0x0007) << 1);
                    		uart_printf("I2c addr: %0x\n",i2cAddr);
                    		status  = CyU3PUsbGetEP0Data(wLength, glEp0Buffer, NULL);
                    		if (status == CY_U3P_SUCCESS)
                    		{
                    			status = CyFxFlashProgI2cTransfer (wIndex, i2cAddr, wLength, glEp0Buffer, CyFalse);
                    			if (status == CY_U3P_SUCCESS){
                    				uart_printf("CyFxFlashProgI2cTransfer OK\n");
                    			} else {
                    				uart_printf("CyFxFlashProgI2cTransfer Failed\n");
                    			}

                    		}
                    		glEp0Buffer[0] = 1;
                    		CyU3PUsbSendEP0Data(1,glEp0Buffer);
                    		break;
                    }

                    break;
                case 0xEE:
                	uart_printf("\tProgrammer Check\n");
                	if ((wValue == 0) && (wLength == 1)) {
                		glEp0Buffer[0] = 0;
                		CyU3PUsbSendEP0Data (1, glEp0Buffer);
                	}
                	break;
                default:
                	uart_printf("unknown request: %d\n",bRequest);
                    /* This is unknown request. */
                    isHandled = CyFalse;
                    break;
            }

            /* If there was any error, return not handled so that the library will
             * stall the request. Alternatively EP0 can be stalled here and return
             * CyTrue. */
            if (status != CY_U3P_SUCCESS)
            {
                //isHandled = CyFalse;
            }
            //return isHandled;
        }

    if (bmType == CY_U3P_USB_CLASS_RQT)
            {
                isHandled = CyTrue;
                if (bRequest == SET_LINE_CODING)
                {
                    status = CyU3PUsbGetEP0Data(0x07, config_data, &readCount);
                    /*if (status != CY_U3P_SUCCESS)
                    {
                        CyFxAppErrorHandler(__LINE__,status);
                    }
                    if (readCount != 0x07)
                    {
                        CyFxAppErrorHandler(__LINE__,CY_U3P_ERROR_BAD_SIZE);
                    }
                    else
                    {
                        CyU3PMemSet ((uint8_t *)&uartConfig, 0, sizeof (uartConfig));
                        uartConfig.baudRate = (CyU3PUartBaudrate_t)(config_data[0] | (config_data[1]<<8)|
                                (config_data[2]<<16)|(config_data[3]<<24));
                        if (config_data[4] == 0)
                        {
                            uartConfig.stopBit = CY_U3P_UART_ONE_STOP_BIT;
                        }
                        else if (config_data[4] == 2)
                        {
                            uartConfig.stopBit = CY_U3P_UART_TWO_STOP_BIT;
                        }
                        else
                        {
                            uartConfig.stopBit = (CyU3PUartStopBit_t)0;
                        }
                        if (config_data[5] == 1)
                        {
                            uartConfig.parity = CY_U3P_UART_ODD_PARITY;
                        }
                        else if (config_data[5] == 2)
                        {
                            uartConfig.parity = CY_U3P_UART_EVEN_PARITY;
                        }
                        else
                        {
                            uartConfig.parity = CY_U3P_UART_NO_PARITY;
                        }

                        uartConfig.txEnable = CyTrue;
                        uartConfig.rxEnable = CyTrue;
                        uartConfig.flowCtrl = CyFalse;
                        uartConfig.isDma = CyTrue;

                        apiRetStatus = CyU3PUartSetConfig (&uartConfig, NULL);
                        if (apiRetStatus == CY_U3P_SUCCESS)
                        {
                            CyU3PMemCopy ((uint8_t *)&glUartConfig, (uint8_t *)&uartConfig,
                                    sizeof (CyU3PUartConfig_t));
                        }
                    }*/
                }

                else if (bRequest == GET_LINE_CODING )
                {
                    config_data[0] = glUartConfig.baudRate&(0x000000FF);
                    config_data[1] = ((glUartConfig.baudRate&(0x0000FF00))>> 8);
                    config_data[2] = ((glUartConfig.baudRate&(0x00FF0000))>>16);
                    config_data[3] = ((glUartConfig.baudRate&(0xFF000000))>>24);
                    if (glUartConfig.stopBit == CY_U3P_UART_ONE_STOP_BIT)
                    {
                        config_data[4] = 0;
                    }
                    else
                    {
                        config_data[4] = 2;
                    }

                    if (glUartConfig.parity == CY_U3P_UART_EVEN_PARITY)
                    {
                        config_data[5] = 2;
                    }
                    else if (glUartConfig.parity == CY_U3P_UART_ODD_PARITY)
                    {
                        config_data[5] = 1;
                    }
                    else
                    {
                        config_data[5] = 0;
                    }
                    config_data[6] =  0x08;
                    status = CyU3PUsbSendEP0Data( 0x07, config_data);
                    if (status != CY_U3P_SUCCESS)
                    {
                        CyFxAppErrorHandler(__LINE__,status);
                    }
                }

                else if (bRequest == SET_CONTROL_LINE_STATE)
                {
                    if (glIsAppUARTlnActive)
                    {
                        CyU3PUsbAckSetup ();
                    }
                    else
                        CyU3PUsbStall (0, CyTrue, CyFalse);
                }
                else
                {
                    status = CY_U3P_ERROR_FAILURE;
                }

                if (status != CY_U3P_SUCCESS)
                {
                    isHandled = CyFalse;
                }
                //return isHandled;
            }


    isHandled = UVCRequestHandler(isHandled, bmReqType, bmType, bRequest, wValue, wIndex, wLength);

    //if (isHandled == CyFalse) uart_printf( "CyFxUVCApplnUSBSetupCB: [%08x %08x]\tbmReqType:%x, \tbmType: %x, \tbRequest:%x, \tbTarget:%x, \tValue:%x, \tLength:%x, \tIndex:%x\r\n", setupdat0, setupdat1, bmReqType, bmType, bRequest,bTarget, wValue,wLength, wIndex);
    return isHandled;
}

/* Callback function to handle LPM requests from the USB 3.0 host. This function is invoked by the API
   whenever a state change from U0 -> U1 or U0 -> U2 happens. If we return CyTrue from this function, the
   FX3 device is retained in the low power state. If we return CyFalse, the FX3 device immediately tries
   to trigger an exit back to U0.

   This application does not have any state in which we should not allow U1/U2 transitions; and therefore
   the function always return CyTrue.
 */
CyBool_t
CyFxApplnLPMRqtCB (
        CyU3PUsbLinkPowerMode link_mode)
{
    return CyFalse;//CyTrue;
}

/* GpifCB callback function is invoked when FV triggers GPIF interrupt */
void
CyFxGpifCB (
        uint8_t currentState            /* GPIF state which triggered the interrupt. */
        )
{
    /* The ongoing video frame has ended. If we have a partial buffer sitting on the socket, we need to forcibly
     * wrap it up. We also need to toggle the FW_TRG a couple of times to get the state machine ready for the
     * next frame.
     *
     * Note: DMA channel APIs cannot be used here as this is ISR context. We are making use of the raw socket
     * APIs.
     */
    switch (currentState)
    {
        case PARTIAL_BUF_IN_SCK0:
            CyU3PDmaSocketSetWrapUp (CY_U3P_PIB_SOCKET_0);
            break;
        case FULL_BUF_IN_SCK0:
            break;
        case PARTIAL_BUF_IN_SCK1:
            CyU3PDmaSocketSetWrapUp (CY_U3P_PIB_SOCKET_1);
            break;
        case FULL_BUF_IN_SCK1:
            break;

        default:
            /* This should not happen. Do nothing. */
            return;
    }

    CyU3PGpifControlSWInput (CyTrue);
    CyU3PGpifControlSWInput (CyFalse);
}

static void
CyFxUvcAppGpifInit (
        void)
{
    CyU3PReturnStatus_t apiRetStatus;

    apiRetStatus =  CyU3PGpifLoad ((CyU3PGpifConfig_t *) &CyFxGpifConfig);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        /* Error Handling */
        uart_printf( "Loading GPIF Configuration failed, Error Code = %d\r\n", apiRetStatus);
        CyFxAppErrorHandler(__LINE__,apiRetStatus);
    }
}
/* This function initializes the USB Module, creates event group,
   sets the enumeration descriptors, configures the Endpoints and
   configures the DMA module for the UVC Application */
void
CyFxUVCApplnInit (void)
{
    CyU3PEpConfig_t endPointConfig;
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;

    /* Start the USB functionality */


    // =============== INIT GPIF ===============
    CyU3PGpioClock_t             gpioClock;
    CyU3PPibClock_t              pibclock;

    apiRetStatus = CyU3PEventCreate (&glFxUVCEvent);
        if (apiRetStatus != 0)
        {
            uart_printf( "UVC Create Event failed, Error Code = %d\n", apiRetStatus);
            CyFxAppErrorHandler(__LINE__,apiRetStatus);
        }
        /* Init the GPIO module */
        gpioClock.fastClkDiv = 2;
        gpioClock.slowClkDiv = 2;
        gpioClock.simpleDiv  = CY_U3P_GPIO_SIMPLE_DIV_BY_2;
        gpioClock.clkSrc     = CY_U3P_SYS_CLK;
        gpioClock.halfDiv    = 0;

        /* Initialize Gpio interface */
        apiRetStatus = CyU3PGpioInit (&gpioClock, NULL);
        if (apiRetStatus != 0)
        {
            uart_printf( "GPIO Init failed, Error Code = %d\n", apiRetStatus);
            CyFxAppErrorHandler(__LINE__,apiRetStatus);
        }


        /* Initialize the P-port. */
        pibclock.clkDiv      = 2;
        pibclock.clkSrc      = CY_U3P_SYS_CLK;
        pibclock.isDllEnable = CyFalse;
        pibclock.isHalfDiv   = CyFalse;

        apiRetStatus = CyU3PPibInit (CyTrue, &pibclock);
        if (apiRetStatus != CY_U3P_SUCCESS)
        {
            uart_printf( "PIB Function Failed to Start, Error Code = %d\n", apiRetStatus);
            CyFxAppErrorHandler(__LINE__,apiRetStatus);
        }

        CyFxUvcAppGpifInit ();

        /* Register the GPIF State Machine callback used to get frame end notifications.
         * We use the fast callback version which is triggered from ISR context.
         */
        CyU3PGpifRegisterSMIntrCallback (CyFxGpifCB);
    // =========== END INIT GPIF ===============

    apiRetStatus = CyU3PUsbStart();
    if (apiRetStatus != CY_U3P_SUCCESS) CyFxAppErrorHandler(__LINE__,apiRetStatus);


    CyU3PUsbRegisterSetupCallback(CyFxUVCApplnUSBSetupCB, CyTrue);
    CyU3PUsbRegisterEventCallback(CyFxUVCApplnUSBEventCB);
    CyU3PUsbRegisterLPMRequestCallback(CyFxApplnLPMRqtCB);    
    
    CyU3PDmaMultiChannelConfig_t dmaMultiConfig;
    dmaMultiConfig.size           = CY_FX_UVC_STREAM_BUF_SIZE;
    dmaMultiConfig.count          = CY_FX_UVC_STREAM_BUF_COUNT;
    dmaMultiConfig.validSckCount  = 2;
    dmaMultiConfig.prodSckId [0]  = (CyU3PDmaSocketId_t)CY_U3P_PIB_SOCKET_0;
    dmaMultiConfig.prodSckId [1]  = (CyU3PDmaSocketId_t)CY_U3P_PIB_SOCKET_1;
    dmaMultiConfig.consSckId [0]  = (CyU3PDmaSocketId_t)(CY_U3P_UIB_SOCKET_CONS_0 | CY_FX_EP_VIDEO_CONS_SOCKET);
    dmaMultiConfig.prodAvailCount = 0;
    dmaMultiConfig.prodHeader     = 12;                 /* 12 byte UVC header to be added. */
    dmaMultiConfig.prodFooter     = 4;                  /* 4 byte footer to compensate for the 12 byte header. */
    dmaMultiConfig.consHeader     = 0;
    dmaMultiConfig.dmaMode        = CY_U3P_DMA_MODE_BYTE;
    dmaMultiConfig.notification   = CY_U3P_DMA_CB_PROD_EVENT | CY_U3P_DMA_CB_CONS_EVENT;
    dmaMultiConfig.cb             = CyFxUvcApplnDmaCallback;
    apiRetStatus = CyU3PDmaMultiChannelCreate (&glChHandleUVCStream, CY_U3P_DMA_TYPE_MANUAL_MANY_TO_ONE, &dmaMultiConfig);
    if (apiRetStatus != CY_U3P_SUCCESS)CyFxAppErrorHandler(__LINE__,apiRetStatus);


    /* Set the USB Enumeration descriptors */

    /* Super speed device descriptor. */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_SS_DEVICE_DESCR, 0, (uint8_t *)CyFxUSB30DeviceDscr);
    if (apiRetStatus != CY_U3P_SUCCESS) CyFxAppErrorHandler(__LINE__,apiRetStatus);
    /* High speed device descriptor. */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_HS_DEVICE_DESCR, 0, (uint8_t *)CyFxUSB20DeviceDscr);
    if (apiRetStatus != CY_U3P_SUCCESS )CyFxAppErrorHandler(__LINE__,apiRetStatus);
    /* BOS descriptor */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_SS_BOS_DESCR, 0, (uint8_t *)CyFxUSBBOSDscr);
    if (apiRetStatus != CY_U3P_SUCCESS) CyFxAppErrorHandler(__LINE__,apiRetStatus);
    /* Device qualifier descriptor */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_DEVQUAL_DESCR, 0, (uint8_t *)CyFxUSBDeviceQualDscr);
    if (apiRetStatus != CY_U3P_SUCCESS) CyFxAppErrorHandler(__LINE__,apiRetStatus);
    /* Super speed configuration descriptor */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_SS_CONFIG_DESCR, 0, (uint8_t *)CyFxUSBSSConfigDscr);
    if (apiRetStatus != CY_U3P_SUCCESS) CyFxAppErrorHandler(__LINE__,apiRetStatus);
    /* High speed configuration descriptor */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_HS_CONFIG_DESCR, 0, (uint8_t *)CyFxUSBHSConfigDscr);
    if (apiRetStatus != CY_U3P_SUCCESS) CyFxAppErrorHandler(__LINE__,apiRetStatus);
    /* Full speed configuration descriptor */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_FS_CONFIG_DESCR, 0, (uint8_t *)CyFxUSBFSConfigDscr);
    if (apiRetStatus != CY_U3P_SUCCESS) CyFxAppErrorHandler(__LINE__,apiRetStatus);
    /* String descriptor 0 */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_STRING_DESCR, 0, (uint8_t *)CyFxUSBStringLangIDDscr);
    if (apiRetStatus != CY_U3P_SUCCESS) CyFxAppErrorHandler(__LINE__,apiRetStatus);
    /* String descriptor 1 */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_STRING_DESCR, 1, (uint8_t *)CyFxUSBManufactureDscr);
    if (apiRetStatus != CY_U3P_SUCCESS) CyFxAppErrorHandler(__LINE__,apiRetStatus);
    /* String descriptor 2 */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_STRING_DESCR, 2, (uint8_t *)CyFxUSBProductDscr);
    if (apiRetStatus != CY_U3P_SUCCESS) CyFxAppErrorHandler(__LINE__,apiRetStatus);

    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_STRING_DESCR, 3, (uint8_t *)CyFxUSBProductDscrProg);
    if (apiRetStatus != CY_U3P_SUCCESS) CyFxAppErrorHandler(__LINE__,apiRetStatus);


    endPointConfig.enable   = 1;
    endPointConfig.epType   = CY_U3P_USB_EP_BULK;
    endPointConfig.pcktSize = CY_FX_EP_BULK_VIDEO_PKT_SIZE;
    endPointConfig.isoPkts  = 1;
    endPointConfig.burstLen = 16;
    endPointConfig.streams  = 0;

    apiRetStatus = CyU3PSetEpConfig(CY_FX_EP_BULK_VIDEO, &endPointConfig);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
    	CyFxAppErrorHandler(__LINE__,apiRetStatus);
    }

    endPointConfig.enable   = 1;
    endPointConfig.epType   = CY_U3P_USB_EP_INTR;
    endPointConfig.pcktSize = 64;
    endPointConfig.isoPkts  = 0;
    endPointConfig.streams  = 0;
    endPointConfig.burstLen = 1;

    apiRetStatus = CyU3PSetEpConfig(CY_FX_EP_CONTROL_STATUS, &endPointConfig);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyFxAppErrorHandler(__LINE__,apiRetStatus);
    }

    apiRetStatus = CyU3PConnectState(CyTrue, CyTrue);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyFxAppErrorHandler(__LINE__,apiRetStatus);
    }
}







void
USBUARTAppThread_Entry(
		uint32_t input)
{



	CyFxUArtInit();
	CyFxFlashProgI2cInit (0x40);

	uint32_t regValueEn = UART->lpp_uart_config;
	uint32_t regValueDs = UART->lpp_uart_config & (~(CY_U3P_LPP_UART_RTS | CY_U3P_LPP_UART_RX_ENABLE));

	uart_printf( "=== USBUARTAppThread_Entry ===\n");
	for (;;) {
#ifndef DEBUG_UART
		if (glIsAppUARTlnActive){
			/*if (glUartPacketsPending == 0) {
				UART->lpp_uart_config = regValueDs;
				CyU3PDmaChannelSetWrapUp (&chHandleUSBSerial_OUT);
				UART->lpp_uart_config = regValueEn;
			}
			glUartPacketsPending = 0;*/
		}
	    CyU3PThreadSleep (50);
#else
	    //uart_printf( "=== USBUARTAppThread_Entry ===\n");
	    if (glIsApplnActive) uart_printf( "UVC: Completed %d frames and %d buffers. Limit reached %d times\r\n", glFrameCount, (glDmaDone != 0) ? (glDmaDone - 1) : 0,gl_commit_count_limit_reached);
	    CyU3PThreadSleep (1000);
#endif
	}

}


/*
 * Handler for control requests addressed to the Processing Unit.
 */
static void
UVCHandleProcessingUnitRqts (
        void)
{
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;
    uint16_t readCount, brightnessVal;

    switch (wValue)
    {
        case CY_FX_UVC_PU_BRIGHTNESS_CONTROL:
            switch (bRequest)
            {
                case CY_FX_USB_UVC_GET_LEN_REQ: /* Length of brightness data = 2 byte. */
                    glEp0Buffer[0] = 2;
                    CyU3PUsbSendEP0Data (2, (uint8_t *)glEp0Buffer);
                    break;
                case CY_FX_USB_UVC_GET_CUR_REQ: /* Current brightness value. */
                    glEp0Buffer[0] = 0;
                    CyU3PUsbSendEP0Data (2, (uint8_t *)glEp0Buffer);
                    break;
                case CY_FX_USB_UVC_GET_MIN_REQ: /* Minimum brightness = 0. */
                    glEp0Buffer[0] = 0;
                    CyU3PUsbSendEP0Data (2, (uint8_t *)glEp0Buffer);
                    break;
                case CY_FX_USB_UVC_GET_MAX_REQ: /* Maximum brightness = 255. */
                    glEp0Buffer[0] = 255;
                    CyU3PUsbSendEP0Data (2, (uint8_t *)glEp0Buffer);
                    break;
                case CY_FX_USB_UVC_GET_RES_REQ: /* Resolution = 1. */
                    glEp0Buffer[0] = 1;
                    CyU3PUsbSendEP0Data (2, (uint8_t *)glEp0Buffer);
                    break;
                case CY_FX_USB_UVC_GET_INFO_REQ: /* Both GET and SET requests are supported, auto modes not supported */
                    glEp0Buffer[0] = 3;
                    CyU3PUsbSendEP0Data (2, (uint8_t *)glEp0Buffer);
                    break;
                case CY_FX_USB_UVC_GET_DEF_REQ: /* Default brightness value = 55. */
                    glEp0Buffer[0] = 55;
                    CyU3PUsbSendEP0Data (2, (uint8_t *)glEp0Buffer);
                    break;
                case CY_FX_USB_UVC_SET_CUR_REQ: /* Update brightness value. */
                    apiRetStatus = CyU3PUsbGetEP0Data (CY_FX_UVC_MAX_PROBE_SETTING_ALIGNED,
                            glEp0Buffer, &readCount);
                    if (apiRetStatus == CY_U3P_SUCCESS)
                    {
                        brightnessVal = CY_U3P_MAKEWORD(glEp0Buffer[1], glEp0Buffer[0]);
                        /* Update the brightness value only if the value is within the range */
                        if(brightnessVal >= 0 && brightnessVal <= 255)
                        {

                        }
                    }
                    break;
                default:
                    glUvcVcErrorCode = CY_FX_UVC_VC_ERROR_CODE_INVALID_REQUEST;
                    CyU3PUsbStall (0, CyTrue, CyFalse);
                    break;
            }
            break;

        default:
            /*
             * Only the brightness control is supported as of now. Add additional code here to support
             * other controls.
             */
            glUvcVcErrorCode = CY_FX_UVC_VC_ERROR_CODE_INVALID_CONTROL;
            CyU3PUsbStall (0, CyTrue, CyFalse);
            break;
    }
}

/*
 * Handler for control requests addressed to the UVC Camera Terminal unit.
 */
static void
UVCHandleCameraTerminalRqts (
        void)
{

    switch (wValue)
    {
        default:
            glUvcVcErrorCode = CY_FX_UVC_VC_ERROR_CODE_INVALID_CONTROL;
            CyU3PUsbStall (0, CyTrue, CyFalse);
            break;
    }
}

/*
 * Handler for UVC Interface control requests.
 */
static void
UVCHandleInterfaceCtrlRqts (
        void)
{
    switch (wValue)
    {
        /* Control to send video control errors to the Host. When device stalls a video
         * control request, Windows host gets the error through this control */
        case CY_FX_UVC_VC_REQUEST_ERROR_CODE_CONTROL:
            switch (bRequest)
            {
                case CY_FX_USB_UVC_GET_CUR_REQ:
                    CyU3PUsbSendEP0Data(1, &glUvcVcErrorCode);
                    glUvcVcErrorCode = CY_FX_UVC_VC_ERROR_CODE_NO_ERROR;
                    break;
            }
            break;
    }
}

/*
 * Handler for control requests addressed to the Extension Unit.
 */
static void
UVCHandleExtensionUnitRqts (
        void)
{

    switch (wValue)
    {
        default:
            glUvcVcErrorCode = CY_FX_UVC_VC_ERROR_CODE_INVALID_CONTROL;
            CyU3PUsbStall (0, CyTrue, CyFalse);
            break;
    }
}


/*
 * Handler for the video streaming control requests.
 */
static void
UVCHandleVideoStreamingRqts (
        void)
{
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;
    uint16_t readCount;

    switch (wValue)
    {
        case CY_FX_UVC_PROBE_CTRL:
        	//uart_printf( "UVCHandleVideoStreamingRqts: CY_FX_UVC_PROBE_CTRL\r\n");
            switch (bRequest)
            {
                case CY_FX_USB_UVC_GET_INFO_REQ:
                    glEp0Buffer[0] = 3;                /* GET/SET requests are supported. */
                    CyU3PUsbSendEP0Data (1, (uint8_t *)glEp0Buffer);
                    break;
                case CY_FX_USB_UVC_GET_LEN_REQ:
                    glEp0Buffer[0] = CY_FX_UVC_MAX_PROBE_SETTING;
                    CyU3PUsbSendEP0Data (1, (uint8_t *)glEp0Buffer);
                    break;
                case CY_FX_USB_UVC_GET_CUR_REQ:
                case CY_FX_USB_UVC_GET_MIN_REQ:
                case CY_FX_USB_UVC_GET_MAX_REQ:
                case CY_FX_USB_UVC_GET_DEF_REQ: /* There is only one setting per USB speed. */
                    if (usbSpeed == CY_U3P_SUPER_SPEED)
                    {
                        CyU3PUsbSendEP0Data (CY_FX_UVC_MAX_PROBE_SETTING, (uint8_t *)glProbeCtrl);
                    }
                    else
                    {
                        CyU3PUsbSendEP0Data (CY_FX_UVC_MAX_PROBE_SETTING, (uint8_t *)glProbeCtrl20);
                    }
                    break;
                case CY_FX_USB_UVC_SET_CUR_REQ:
                    apiRetStatus = CyU3PUsbGetEP0Data (CY_FX_UVC_MAX_PROBE_SETTING_ALIGNED,
                            glCommitCtrl, &readCount);
                    if (apiRetStatus == CY_U3P_SUCCESS)
                    {
                        /* Copy the relevant settings from the host provided data into the
                           active data structure. */
                        glProbeCtrl[2] = glCommitCtrl[2];
                        glProbeCtrl[3] = glCommitCtrl[3];
                        glProbeCtrl[4] = glCommitCtrl[4];
                        glProbeCtrl[5] = glCommitCtrl[5];
                        glProbeCtrl[6] = glCommitCtrl[6];
                        glProbeCtrl[7] = glCommitCtrl[7];
                    }
                    break;
                default:
                	//uart_printf( "UVCHandleVideoStreamingRqts: unknown bRequest(0x%x)\r\n",bRequest);
                    CyU3PUsbStall (0, CyTrue, CyFalse);
                    break;
            }
            break;

        case CY_FX_UVC_COMMIT_CTRL:
        	//uart_printf( "UVCHandleVideoStreamingRqts: CY_FX_UVC_COMMIT_CTRL\r\n");
            switch (bRequest)
            {
                case CY_FX_USB_UVC_GET_INFO_REQ:
                    glEp0Buffer[0] = 3;                        /* GET/SET requests are supported. */
                    CyU3PUsbSendEP0Data (1, (uint8_t *)glEp0Buffer);
                    break;
                case CY_FX_USB_UVC_GET_LEN_REQ:
                    glEp0Buffer[0] = CY_FX_UVC_MAX_PROBE_SETTING;
                    CyU3PUsbSendEP0Data (1, (uint8_t *)glEp0Buffer);
                    break;
                case CY_FX_USB_UVC_GET_CUR_REQ:
                    if (usbSpeed == CY_U3P_SUPER_SPEED)
                    {
                        CyU3PUsbSendEP0Data (CY_FX_UVC_MAX_PROBE_SETTING, (uint8_t *)glProbeCtrl);
                    }
                    else
                    {
                        CyU3PUsbSendEP0Data (CY_FX_UVC_MAX_PROBE_SETTING, (uint8_t *)glProbeCtrl20);
                    }
                    break;
                case CY_FX_USB_UVC_SET_CUR_REQ:
                    /* The host has selected the parameters for the video stream. Check the desired
                       resolution settings, configure the sensor and start the video stream.
                       */
                    apiRetStatus = CyU3PUsbGetEP0Data (CY_FX_UVC_MAX_PROBE_SETTING_ALIGNED,
                            glCommitCtrl, &readCount);
                    if (apiRetStatus == CY_U3P_SUCCESS)
                    {

                        /* We can start streaming video now. */
                        apiRetStatus = CyU3PEventSet (&glFxUVCEvent, CY_FX_UVC_STREAM_EVENT, CYU3P_EVENT_OR);
                        if (apiRetStatus != CY_U3P_SUCCESS)
                        {
                            uart_printf( "Set CY_FX_UVC_STREAM_EVENT failed %x\r\n", apiRetStatus);
                        }
                    }
                    break;

                default:
                	//uart_printf( "UVCHandleVideoStreamingRqts: unknown bRequest(0x%x)\r\n",bRequest);
                	CyU3PUsbStall (0, CyTrue, CyFalse);
                    break;
            }
            break;

        default:
        	//uart_printf( "UVCHandleVideoStreamingRqts: unknown (0x%x)\r\n",wValue);
            CyU3PUsbStall (0, CyTrue, CyFalse);
            break;
    }
}


void reset_dma_in_case_of_invalid_sequence() {
	/* Place the Endpoint in NAK mode before cleaning up the pipe. */


	CyU3PGpifDisable (CyFalse);

	int apiRetStatus = CyU3PUsbSetEpNak (CY_FX_EP_BULK_VIDEO, CyTrue);

	if (apiRetStatus != CY_U3P_SUCCESS)
	{
	        uart_printf("\r EP_NAK=%d\n", apiRetStatus);
	}


	/* Reset the DMA channel and flush the endpoint pipe. */
	apiRetStatus = CyU3PDmaMultiChannelReset (&glChHandleUVCStream);
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		uart_printf( "\r DMA_RESET=%d\n", apiRetStatus);
	 }

	apiRetStatus = CyU3PUsbFlushEp (CY_FX_EP_BULK_VIDEO);
	if(apiRetStatus != CY_U3P_SUCCESS)
	{
		uart_printf( "\r EP_FLUSH=%d\n", apiRetStatus);
	}

	apiRetStatus = CyU3PDmaMultiChannelSetXfer (&glChHandleUVCStream, 0, 0);
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		uart_printf( "\r DMA_XFER  = %d\n", apiRetStatus);
	}

	apiRetStatus = CyU3PUsbSetEpNak (CY_FX_EP_BULK_VIDEO, CyFalse);
	if(apiRetStatus != CY_U3P_SUCCESS)
	{
		uart_printf( "\r EP_ACK=%d\n", apiRetStatus);
	}


	apiRetStatus = CyU3PGpifSMStart (START_SCK0, ALPHA_START_SCK0);
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		uart_printf( "CyU3PGpifSMStart  = %d\n", apiRetStatus);
	}


	apiRetStatus = CyU3PGpifSMSwitch (257, 0, 257, 0, 2);
	if (apiRetStatus != CY_U3P_SUCCESS)
	{
		uart_printf( "CyU3PGpifSMSwitch  = %d\n", apiRetStatus);
	}

	gl_commit_count=0;
	gl_commit_count_limit_reached = 0;
}

/* Entry function for the UVC application thread. */
void
UVCAppThread_Entry (
        uint32_t input)
{
	CyU3PUsbLinkPowerMode usb3mode;
	uint16_t              wakeReason;
	CyU3PReturnStatus_t   apiRetStatus;
	uint32_t              flag;



    /* Initialize the UVC Application */
    CyFxUVCApplnInit();
    CyFxUArtInit();
    CyFxFlashProgI2cInit (0x40);

    uint32_t regValueEn = UART->lpp_uart_config;
    uint32_t regValueDs = UART->lpp_uart_config & (~(CY_U3P_LPP_UART_RTS | CY_U3P_LPP_UART_RX_ENABLE));

    for (;;)
        {
            apiRetStatus = CyU3PEventGet (&glFxUVCEvent,
            		CY_FX_UVC_STREAM_ABORT_EVENT |
            		CY_FX_UVC_STREAM_EVENT |
            		CY_FX_UVC_VIDEO_CONTROL_REQUEST_EVENT |
            		CY_FX_UVC_VIDEO_STREAM_REQUEST_EVENT |
                    CY_FX_UVC_DMA_RESET_EVENT |
                    CY_FX_USB_SUSPEND_EVENT_HANDLER,
                    CYU3P_EVENT_OR_CLEAR, &flag, 1000);

            if (apiRetStatus == CY_U3P_SUCCESS)
            {
            	//uart_printf( "CyU3PEventGet: %x\n",flag);
                /* Request to start video stream. */
                if ((flag & CY_FX_UVC_STREAM_EVENT) != 0)
                {
                    glIsApplnActive = CyTrue;
                    CyFxUVCApplnStart();
                }

                /* Video stream abort requested. */
                if ((flag & CY_FX_UVC_STREAM_ABORT_EVENT) != 0)
                {
                    glIsApplnActive = CyFalse;
                    CyFxUVCApplnStop();
                }

                if (((flag & CY_FX_UVC_DMA_RESET_EVENT) != 0) && glIsApplnActive)
                {
                    //if(glDmaResetFlag == CY_FX_UVC_DMA_RESET_COMMIT_BUFFER_FAILURE)
                    uart_printf( "\n\t ======== DMA Reset Event: Commit buffer failure (%x) ======== \n\n",glDmaResetResult);

                    if (glDmaResetResult == CY_U3P_ERROR_INVALID_SEQUENCE) {
                    	reset_dma_in_case_of_invalid_sequence();
                    } else {
                    	glIsApplnActive = CyFalse;
                    	CyFxUVCApplnStop();

                    	if ((glIsApplnActive) && (++glCommitBufferFailureCount < CY_FX_UVC_MAX_COMMIT_BUF_FAILURE_CNT))
                    	{
                    		glIsApplnActive = CyTrue;
                    		CyFxUVCApplnStart();
                    	}

                    	if(glCommitBufferFailureCount == CY_FX_UVC_MAX_COMMIT_BUF_FAILURE_CNT)
                    	{
                    		glIsApplnActive = CyFalse;
                    		uart_printf( "Application Stopped after %d Commit buffer failures\r\n", glCommitBufferFailureCount);
                    		glCommitBufferFailureCount = 0;
                    	}
                    }
                    glDmaResetResult = 0;
                }

                /* Handle USB suspend event by putting device into low power mode */
                if ((flag & CY_FX_USB_SUSPEND_EVENT_HANDLER) != 0)
                {
                    /* Include your code here... to reset and/or shutdown power to the sensor/ISP
                     * This will help to reduce the overall power consumption by the kit */

                    /* Place FX3 in Low Power Suspend mode */
                    uart_printf( "Entering USB Suspend Mode\r\n");
                    CyU3PThreadSleep(5);

                    /* As per the USB3 specs, link layer takes 10ms (Max.) to leave U3 state. If the device sees some spurious (unwanted)
                     * USB activity it will leave suspend mode (even though USB is in U3 state). The device can wait for 10ms (U3 exit LFPS duration)
                     * and check the Link Layer State. If the link layer is not in U3 state, CX3 device will wake up else it will enter suspend mode */
                    do
                    {
                        apiRetStatus = CyU3PSysEnterSuspendMode(CY_U3P_SYS_USB_BUS_ACTVTY_WAKEUP_SRC, 0, &wakeReason);
                        if ((apiRetStatus != CY_U3P_SUCCESS) || (CyU3PUsbGetSpeed() != CY_U3P_SUPER_SPEED))
                            break;

                        /* Wait for the maximum U3 exit LFPS duration. */
                        CyU3PThreadSleep(10);

                        /* If the link is still in U3, we can continue to attempt Suspend mode entry. */
                        apiRetStatus = CyU3PUsbGetLinkPowerState(&usb3mode);
                        if ((apiRetStatus != CY_U3P_SUCCESS) || (usb3mode != CyU3PUsbLPM_U3))
                            break;
                    }while(1);

                    /* Leaving Low Power Suspend mode */
                    uart_printf( "Leaving Suspend Mode\r\n");

                    /* Include your code here... to bring sensor/ISP out of reset and/or switch ON the power to the sensor/ISP */
                }

                if (flag & CY_FX_UVC_VIDEO_CONTROL_REQUEST_EVENT) {
                	switch ((wIndex >> 8)) {
                		case CY_FX_UVC_PROCESSING_UNIT_ID:
                			UVCHandleProcessingUnitRqts ();
                			break;
                		case CY_FX_UVC_CAMERA_TERMINAL_ID:
                            UVCHandleCameraTerminalRqts ();
                            break;
                        case CY_FX_UVC_INTERFACE_CTRL:
                        	UVCHandleInterfaceCtrlRqts ();
                            break;
                        case CY_FX_UVC_EXTENSION_UNIT_ID:
                        	UVCHandleExtensionUnitRqts ();
                            break;
                        default:
                        	CyU3PUsbStall (0, CyTrue, CyFalse);
                            break;
                    }
                 }

                if (flag & CY_FX_UVC_VIDEO_STREAM_REQUEST_EVENT) {
                	if (wIndex != CY_FX_UVC_STREAM_INTERFACE) {
                		uart_printf( "\t\twIndex != CY_FX_UVC_STREAM_INTERFACE: \t%d != %d\n", wIndex,CY_FX_UVC_STREAM_INTERFACE);
                        CyU3PUsbStall (0, CyTrue, CyFalse);
                    } else {
                        UVCHandleVideoStreamingRqts ();
                    }
                }
            }
#ifndef DEBUG_UART
		if (glIsAppUARTlnActive){
			if (glUartPacketsPending == 0) {
				UART->lpp_uart_config = regValueDs;
				CyU3PDmaChannelSetWrapUp (&chHandleUSBSerial_OUT);
				UART->lpp_uart_config = regValueEn;
			}
			glUartPacketsPending = 0;
		}
	    //CyU3PThreadSleep (50);
#else
	    //uart_printf( "=== USBUARTAppThread_Entry ===\n");
	    if (glIsApplnActive) uart_printf( "UVC: Completed %d frames and %d buffers. Limit reached %d times\r\n", glFrameCount, (glDmaDone != 0) ? (glDmaDone - 1) : 0,gl_commit_count_limit_reached);
	    //CyU3PThreadSleep (300);
#endif


    //#ifdef DEBUG_UART
            //if (glIsApplnActive) uart_printf( "UVC: Completed %d frames and %d buffers\r\n", glFrameCount, (glDmaDone != 0) ? (glDmaDone - 1) : 0);
    //#endif
        }
}


void
UVCAppEP0Thread_Entry (
        uint32_t input)
{
	//CyU3PThreadSleep (3000);
    uint32_t eventMask = (CY_FX_UVC_VIDEO_CONTROL_REQUEST_EVENT | CY_FX_UVC_VIDEO_STREAM_REQUEST_EVENT);
    uint32_t eventFlag;

    for (;;)
    {
        /* Wait for a Video control or streaming related request on the control endpoint. */
        if (CyU3PEventGet (&glFxUVCEvent, eventMask, CYU3P_EVENT_OR_CLEAR, &eventFlag,
                    CYU3P_WAIT_FOREVER) == CY_U3P_SUCCESS)
        {
            usbSpeed = CyU3PUsbGetSpeed ();

            if (eventFlag & CY_FX_UVC_VIDEO_CONTROL_REQUEST_EVENT)
            {
                switch ((wIndex >> 8))
                {
                    case CY_FX_UVC_PROCESSING_UNIT_ID:
                        UVCHandleProcessingUnitRqts ();
                        break;

                    case CY_FX_UVC_CAMERA_TERMINAL_ID:
                        UVCHandleCameraTerminalRqts ();
                        break;

                    case CY_FX_UVC_INTERFACE_CTRL:
                        UVCHandleInterfaceCtrlRqts ();
                        break;

                    case CY_FX_UVC_EXTENSION_UNIT_ID:
                        UVCHandleExtensionUnitRqts ();
                        break;

                    default:
                        /* Unsupported request. Fail by stalling the control endpoint. */
                        CyU3PUsbStall (0, CyTrue, CyFalse);
                        break;
                }
            }

            if (eventFlag & CY_FX_UVC_VIDEO_STREAM_REQUEST_EVENT)
            {
                if (wIndex != CY_FX_UVC_STREAM_INTERFACE)
                {
                	uart_printf( "\t\twIndex != CY_FX_UVC_STREAM_INTERFACE: \t%d != %d\n", wIndex,CY_FX_UVC_STREAM_INTERFACE);
                    CyU3PUsbStall (0, CyTrue, CyFalse);
                }
                else
                {
                    UVCHandleVideoStreamingRqts ();
                }
            }

        }

        /* Allow other ready threads to run. */
        CyU3PThreadRelinquish ();
    }
}

/* Application define function which creates the threads. */
void
CyFxApplicationDefine (
        void)
{
    void *ptr = NULL, *ptr1 = NULL, *ptr2 = NULL;
    uint32_t retThrdCreate = CY_U3P_SUCCESS;

    /* Allocate the memory for the thread and create the thread */
    ptr = CyU3PMemAlloc (UVC_APP_THREAD_STACK);
    ptr1 = CyU3PMemAlloc (CY_FX_USBUART_THREAD_STACK);
    ptr2 = CyU3PMemAlloc (UVC_APP_THREAD_STACK);

    retThrdCreate = CyU3PThreadCreate (&uvcAppThread,   /* UVC Thread structure */
                           "30:UVC_app_thread",         /* Thread Id and name */
                           UVCAppThread_Entry,          /* UVC Application Thread Entry function */
                           0,                           /* No input parameter to thread */
                           ptr,                         /* Pointer to the allocated thread stack */
                           UVC_APP_THREAD_STACK,        /* UVC Application Thread stack size */
                           UVC_APP_THREAD_PRIORITY,     /* UVC Application Thread priority */
                           UVC_APP_THREAD_PRIORITY,     /* Pre-emption threshold */
                           CYU3P_NO_TIME_SLICE,         /* No time slice for the application thread */
                           CYU3P_AUTO_START             /* Start the Thread immediately */
                           );

    /* Check the return code */
    if (retThrdCreate != 0)
    {
        while(1);
    }
/*
    retThrdCreate = CyU3PThreadCreate (&USBUARTAppThread,
							"21:USBUART_DMA_mode",
							USBUARTAppThread_Entry,
							0,
							ptr1,
							CY_FX_USBUART_THREAD_STACK,
							CY_FX_USBUART_THREAD_PRIORITY,
							CY_FX_USBUART_THREAD_PRIORITY,
							CYU3P_NO_TIME_SLICE,
							CYU3P_AUTO_START
							);
    if (retThrdCreate != 0)
	{
		while(1);
	}

    retThrdCreate = CyU3PThreadCreate (&uvcAppEP0Thread,
                "31:UVC App EP0 Thread",
                UVCAppEP0Thread_Entry,
                0,
                ptr2,
                UVC_APP_THREAD_STACK,
                UVC_APP_THREAD_PRIORITY,
                UVC_APP_THREAD_PRIORITY,
                CYU3P_NO_TIME_SLICE,
                CYU3P_AUTO_START
                );
        if (retThrdCreate != 0)
        {
        	while(1);
        }
*/
}

/*
 * Main function
 */
int
main (void)
{

    CyU3PIoMatrixConfig_t io_cfg;
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
    CyU3PSysClockConfig_t clockConfig;
    clockConfig.setSysClk400  = CyTrue;
    clockConfig.cpuClkDiv     = 2;
    clockConfig.dmaClkDiv     = 2;
    clockConfig.mmioClkDiv    = 2;
    clockConfig.useStandbyClk = CyFalse;
    clockConfig.clkSrc         = CY_U3P_SYS_CLK;

    MARKER[0] = 'A';

    /* Initialize the device */
    status = CyU3PDeviceInit (&clockConfig);
    if (status != CY_U3P_SUCCESS)
    {
        goto handle_fatal_error;
    }

    /* Initialize the caches. Enable only the instruction cache. */
    status = CyU3PDeviceCacheControl (CyTrue, CyFalse, CyFalse);
    if (status != CY_U3P_SUCCESS)
    {
        goto handle_fatal_error;
    }

    /* Configure the IO matrix for the device. On the FX3 DVK board, the COM port 
     * is connected to the IO(53:56). This means that either DQ32 mode should be
     * selected or lppMode should be set to UART_ONLY. Here we are choosing
     * UART_ONLY configuration. */
    io_cfg.isDQ32Bit = CyTrue;
    io_cfg.s0Mode = CY_U3P_SPORT_INACTIVE;
    io_cfg.s1Mode = CY_U3P_SPORT_INACTIVE;
    io_cfg.useUart   = CyTrue;
    io_cfg.useI2C    = CyTrue;
    io_cfg.useI2S    = CyFalse;
    io_cfg.useSpi    = CyFalse;
    io_cfg.lppMode   = CY_U3P_IO_MATRIX_LPP_DEFAULT;
    /* No GPIOs are enabled. */
    io_cfg.gpioSimpleEn[0]  = 0;
    io_cfg.gpioSimpleEn[1]  = 0;
    io_cfg.gpioComplexEn[0] = 0;
    io_cfg.gpioComplexEn[1] = 0;
    status = CyU3PDeviceConfigureIOMatrix (&io_cfg);
    if (status != CY_U3P_SUCCESS)
    {
        goto handle_fatal_error;
    }

    /* This is a non returnable call for initializing the RTOS kernel */
    CyU3PKernelEntry ();

    /* Dummy return to make the compiler happy */
    return 0;

handle_fatal_error:

    /* Cannot recover from this error. */
    while (1);
}

/* [ ] */

